using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Web;
using System.Web.Security;
using System.Xml;
using System.Text;
using System.IO;
using System.Xml.XPath;
using System.Diagnostics;
using System.Xml.Xsl;

using Assetlink.Solutions;
using Assetlink.Solutions.Facade.CommonFacade;
using Assetlink.Widgets.WidgetCommon;

namespace Assetlink.Solutions.Facade.DAMFacade
{
    using ERRORXML = System.String;
    public class DAMHelper : DamFacadeBase
    {
        #region Private members
        ALSolutionCommon ALSolutionCommonObj = null;
        SearchConfiguration obj_Search       = null;
        #endregion

        public String SectionTreeDisplayConfiguration   = null;
        public bool AllowDownloadOfExpiredAsset         = false;
        public String BreadCrumbControlInputFilePath    = null;

        public DAMHelper(String SessionID, String UserID,
                    String TenantName, String CurrentLanguage)
            : base(SessionID, UserID, TenantName, CurrentLanguage)
        {

        }

        public DAMHelper(String SessionID, String UserID,
                String TenantName, String CurrentLanguage, String SolutionName, String ThemeName)
            : base(SessionID, UserID, TenantName, CurrentLanguage, SolutionName,ThemeName)
        {

        }

        # region Public methods

        /// <alsummary>
        /// <createddate> 27 June 2007</createddate>
        /// <author>Mr. Pravin Shinde</author>
        /// <purpose>To get show error page url for session invalid error</purpose>
        ///<param name="ShowErrorPageUrl"></param>
        /// </alsummary>
        public void GetShowErrorPageUrlForSessionInvalid(ref String ShowErrorPageUrl)
        {
            StringBuilder SBShowErrorPageUrl = new StringBuilder();
            ALSolutionCommonObj = new ALSolutionCommon();
            SBShowErrorPageUrl.AppendFormat("/{0}/Common/ASPX/ShowError.aspx?", TenantName);
            SBShowErrorPageUrl.AppendFormat("SessionID={0}&UserID={1}&TenantName={2}&Language={3}", SessionID, UserID, TenantName, CurrentLanguage);
            SBShowErrorPageUrl.Append("&ErrorCode=ASSETLINK_INVALID_SESSION&ErrorMessage=ASSETLINK_INVALID_SESSION");
            SBShowErrorPageUrl.AppendFormat("&ThemeName={0}", ThemeName);
            ShowErrorPageUrl = ALSolutionCommonObj.EncodeURLForResponseRedirects(SBShowErrorPageUrl.ToString());
        }


        /// <alsummary>
        /// <createddate> 5 July 2007</createddate>
        /// <author>Mr. Pravin Shinde</author>
        /// <purpose>To check sections present into the system or not.It checks for root section of non-system vault</purpose>
        /// </alsummary>
        public bool IsRootSectionsAvilable()
        {
            XmlDocument     SearchConfigXmlDoc          = null;
            XmlDocument     SearchInputXmlDoc           = null;
            XmlNode         ConfigurationNode           = null;
            XPathDocument   SearchResultXPathDoc        = null;
            XPathNavigator  SearchResultXPathRootNav    = null;
            String          SearchResultXML             = null;
            String          ErrorXml                    = null;
            try
            {
                String SolnSearchConfigurationXmlPath = ALSolutionCommon.GetConfigurationXMLPath(TenantName, SolutionName, "", "SearchConfiguration");

                SearchConfigXmlDoc = new XmlDocument();
                SearchInputXmlDoc = new XmlDocument();
                SearchConfigXmlDoc.Load(SolnSearchConfigurationXmlPath);
                ConfigurationNode = SearchConfigXmlDoc.SelectSingleNode("//Configuration[@ID='GetSectionInformation']");

                SearchConfiguration SearchConfigurationObj = new SearchConfiguration(SessionID, UserID, TenantName, CurrentLanguage);
                SearchConfigurationObj.PrepareSearchInputXml(ConfigurationNode, null, SearchInputXmlDoc);

                ErrorXml = SearchConfigurationObj.DoSearch(SearchInputXmlDoc.OuterXml, ref SearchResultXML);

                if (!String.IsNullOrEmpty(SearchResultXML))
                {
                    SearchResultXPathDoc = new XPathDocument(new StringReader(SearchResultXML));
                    SearchResultXPathRootNav = SearchResultXPathDoc.CreateNavigator();

                    if (SearchResultXPathRootNav.SelectSingleNode("//SECTION") == null)
                    {
                        return false;
                    }
                    else
                    {
                        return true;
                    }
                }
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
            return true;
        }

        /// <alsummary>
        /// <createddate> 13 Oct 2007</createddate>
        /// <author>Meenal Joshi</author>
        /// <purpose>Get sort arguments like sort order, sort column and sort datatype according to preferences</purpose>
        /// </alsummary>
        public String GetSortArguments(ref String SortOn,ref String SortOrder, ref String SortDataType)
        {
            # region LocalVariables
            String UserPrefrencesXmlPath = null;
            String AppPerferncesXmlPath = null;
            XPathDocument PerferncesXpathDoc = null;
            XPathNavigator PerferncesXpathRootNav = null;
            XPathNavigator PerferncesXpathNav = null;
            String TempFilePath = null;
            String Layout = null;           
            String ListRowCount = null;
            String ListColCount = null;
            String GridRowCount = null;
            String GridColCount = null;
            String ErrorXml = null; 
            
            #endregion

            # region Get Prefernces Xmls
            TempFilePath = ALSolutionCommon.GetTempPath(SessionID);
            UserPrefrencesXmlPath = TempFilePath + UserID + "_DAM_User_Preferences.xml";
            AppPerferncesXmlPath = ALSolutionCommon.GetConfigurationXMLPath(TenantName, "DAM", "", "Application_Preferences");

            #endregion

            try
            {
                # region Select Prefrences

                if (!File.Exists(UserPrefrencesXmlPath.ToString()))
                    GetUserPersonalization();               

                if (System.IO.File.Exists(UserPrefrencesXmlPath.ToString()))
                {                    
                    PerferncesXpathDoc = new XPathDocument(UserPrefrencesXmlPath.ToString());
                }
                else
                    if (System.IO.File.Exists(AppPerferncesXmlPath.ToString()))
                    {
                        PerferncesXpathDoc = new XPathDocument(AppPerferncesXmlPath.ToString());
                    }
                #endregion

                # region Get Settings
                if (PerferncesXpathDoc != null)
                {
                    PerferncesXpathRootNav = PerferncesXpathDoc.CreateNavigator();
                    //Layout 
                    PerferncesXpathNav = PerferncesXpathRootNav.SelectSingleNode("//DISPLAYLAYOUT/LAYOUT");
                    Layout = PerferncesXpathNav.Value;
                    //List Row Count
                    PerferncesXpathNav = PerferncesXpathRootNav.SelectSingleNode("//DISPLAYLAYOUT/LISTROWCOUNT");
                    ListRowCount = PerferncesXpathNav.Value;
                    //List Column Count
                    PerferncesXpathNav = PerferncesXpathRootNav.SelectSingleNode("//DISPLAYLAYOUT/LISTCOLUMNCOUNT");
                    ListColCount = PerferncesXpathNav.Value;
                    //Grid Row Count
                    PerferncesXpathNav = PerferncesXpathRootNav.SelectSingleNode("//DISPLAYLAYOUT/GRIDROWCOUNT");
                    GridRowCount = PerferncesXpathNav.Value;
                    //Grid Column Count
                    PerferncesXpathNav = PerferncesXpathRootNav.SelectSingleNode("//DISPLAYLAYOUT/GRIDCOLUMNCOUNT");
                    GridColCount = PerferncesXpathNav.Value;
                    //SortOn
                    PerferncesXpathNav = PerferncesXpathRootNav.SelectSingleNode("//SORTCONFIGURATION/SORTON");
                    SortOn = PerferncesXpathNav.Value;
                    //Sort Order
                    PerferncesXpathNav = PerferncesXpathRootNav.SelectSingleNode("//SORTCONFIGURATION/SORTORDER");
                    SortOrder = PerferncesXpathNav.Value;
                    //Sort DataType 
                    PerferncesXpathNav = PerferncesXpathRootNav.SelectSingleNode("//SORTCONFIGURATION/SORTDATATYPE");
                    SortDataType = PerferncesXpathNav.Value;                    
                }
                #endregion
            }
            catch (Exception ObjException)
            {
                ErrorXml = ALSolutionCommon.PrepareErrorXML(ObjException.Message);
                return ErrorXml;
            }
            return ErrorXml;
        }

        void GetUserPersonalization()
        {

            String UserPersonalisationXml = null;
            String TempFilePath = null;
            String UserPrefrencesXmlPath = null;
            XmlDocument UserPrefrencesXmlDoc = null;
            XmlNode UserPrefrencesXmlNode = null;

            DAM_Preferences ObjDAM_Preferences = new DAM_Preferences(SessionID, UserID, TenantName, CurrentLanguage);
            TempFilePath = ALSolutionCommon.GetTempPath(SessionID);
            UserPrefrencesXmlPath = TempFilePath + UserID + "_DAM_User_Preferences.xml";
            ObjDAM_Preferences.SolutionName = "DAM";
            String ErrorXml = ObjDAM_Preferences.GetDAMUserPesonalization(ref UserPersonalisationXml);
            if (!String.IsNullOrEmpty(UserPersonalisationXml))
            {
                UserPrefrencesXmlDoc = new XmlDocument();
                UserPrefrencesXmlDoc.LoadXml(UserPersonalisationXml);
                UserPrefrencesXmlNode = UserPrefrencesXmlDoc.SelectSingleNode("descendant::USER_DAMPERSONALISATION/DAMPERSONALIZATION/CONFIGURATIONXML");
                if (UserPrefrencesXmlNode != null)
                {
                    if (!String.IsNullOrEmpty(UserPrefrencesXmlNode.InnerXml))
                    {
                        UserPrefrencesXmlDoc.Save(UserPrefrencesXmlPath);
                    }
                }
            }
        }       

        private bool GetSubsectionList(XmlNode ConfigNode, String ParentSectionId, String GetSectionID, DAM_SectionFacad ObjSection,XPathNavigator xpnav_SubSectionNav, ref String SubsectionTree)
        {
            #region Member Varibales
            String              tempTreeXml     = "";                      
            String              SearchError     = "";
            XPathDocument       tDoc            = null;
            XPathNavigator      SubTreeNode     = null;
            XPathNavigator      xpnav_SectionResult = null;
            StringBuilder       sb_TempResult   = null;
            #endregion

            sb_TempResult       = new StringBuilder(1000);
            xpnav_SectionResult = xpnav_SubSectionNav.SelectSingleNode("SECTIONCOLLECTION/SECTION/SECTIONID[text()='" + ParentSectionId +"']");
            if (xpnav_SectionResult != null)
            {
                xpnav_SectionResult.MoveToParent(); //SECTION
                sb_TempResult.AppendFormat("<SECTIONCOLLECTION>{0}</SECTIONCOLLECTION>", xpnav_SectionResult.OuterXml);

                ObjSection.ShowSectionAssetCount    = ShowSectionAssetCount;
                ObjSection.FormattingXSLTPath       = FormattingXSLTPath;
                SearchError                         = ObjSection.GetSubSectionTree(ConfigNode, sb_TempResult.ToString(), ref tempTreeXml);
                if (String.IsNullOrEmpty(SearchError) && !(String.IsNullOrEmpty(tempTreeXml)))
                {
                    tDoc            = new XPathDocument(new StringReader(tempTreeXml));
                    SubTreeNode     = tDoc.CreateNavigator();
                    SubTreeNode     = SubTreeNode.SelectSingleNode("descendant::TreeNodes");
                    if (SubTreeNode != null)
                        SubsectionTree = SubTreeNode.InnerXml;

                    if (tempTreeXml.IndexOf(GetSectionID) != -1)
                        return true;
                }
            }
            return false;
        }
        /// <summary>
        /// This function gets execute in case of Navigate to Parent Section case
        /// </summary>
        /// <param name="ParentSectionPath"></param>
        /// <param name="SubsectionList"></param>
        /// <returns></returns>
        private Hashtable GetIDsOfParentSection(String ParentSectionPath,ref String SubsectionList)
        {
            #region Member Varibales
            SearchConfiguration         ObjSearch           = null;
            XPathDocument               SearchDocument      = null;            
            String                      SearchResult        = "";
            String                      SearchError         = "";
            Hashtable                   SectionPathList     = null;
            Hashtable                   ParentSectionIDList = null;
            String                      tempSectionPath     = "";
            String                      CurrentSectionPath  = "";
            int                         ParentLevel         = 1;
            StringBuilder               sb_SearchInput      = null;
            StringBuilder               sb_PathListToSearch = null;
            String                      temp_SectionPath    = null;
            String                      Seperator           = null;
            XmlNode                     EmptyNode           = null;
            XPathNavigator              xpNav_SearchRoot    = null;
            XPathNavigator              xpNav_Section       = null;            
            #endregion

            SectionPathList     = new Hashtable();
            ParentSectionIDList = new Hashtable();
            tempSectionPath     = ParentSectionPath;
            ParentLevel         = 1;
            sb_SearchInput      = new StringBuilder(1000);
            sb_PathListToSearch = new StringBuilder(1000);

            #region Prepare SectionPath List
            while (!String.IsNullOrEmpty(tempSectionPath))
            {
                if (tempSectionPath.IndexOf("\\") != -1)
                {
                    if (ParentLevel == 1)
                        CurrentSectionPath = tempSectionPath.Substring(0, tempSectionPath.IndexOf("\\"));
                    else
                        CurrentSectionPath = CurrentSectionPath + "\\" + tempSectionPath.Substring(0, tempSectionPath.IndexOf("\\"));
                    tempSectionPath = tempSectionPath.Substring(tempSectionPath.IndexOf("\\") + 1);
                }
                else
                {
                    if (ParentLevel == 1)
                        CurrentSectionPath = tempSectionPath;
                    else
                        CurrentSectionPath = CurrentSectionPath + "\\" + tempSectionPath;
                    tempSectionPath = "";
                }
                SectionPathList.Add(ParentLevel, CurrentSectionPath);
                ParentLevel++;
            }
            #endregion

            #region Get SectionId and Path of selected List
            Seperator = "";
            for (int FilterCnt = 1; FilterCnt <= SectionPathList.Count; FilterCnt++)
            {
                temp_SectionPath = SectionPathList[FilterCnt].ToString();
                temp_SectionPath = temp_SectionPath.Replace("'", "''");
                sb_PathListToSearch.AppendFormat("{0}'{1}'", Seperator, ALCommon.TrimAndReplaceSpecialChars(temp_SectionPath));
                Seperator = ",";
            }

            sb_SearchInput.Append("<SEARCH>");
            sb_SearchInput.AppendFormat("<SESSIONINFO TenantName='{0}' Language='{1}' UserID='{2}' SessionID='{3}'/>", TenantName, CurrentLanguage, UserID, SessionID);
            sb_SearchInput.AppendFormat(@"<TARGETOBJECT>SECTION</TARGETOBJECT>
                                          <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>                                          
                                          <TARGETENTITY>
	                                        <PRIVILEGENAME>READ</PRIVILEGENAME>	                                       
	                                        <FILTERLIST>		                                        
		                                        <FILTER>
			                                        <FILTERID>1</FILTERID>
			                                        <EXPRESSION>PATH</EXPRESSION>
			                                        <OPERATOR>IN</OPERATOR>
			                                        <EXPRESSIONVALUE>({0})</EXPRESSIONVALUE>
			                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
		                                        </FILTER>                                               
	                                        </FILTERLIST>
                                            <RELATIONSHIP NAME='SECTION_SUBSECTION'>
                                                <FILTERLIST>		                                        
                                                    <FILTER>
			                                            <FILTERID>2</FILTERID>
			                                            <EXPRESSION>MARKEDASDELETED</EXPRESSION>
			                                            <OPERATOR>EQUALTO</OPERATOR>
			                                            <EXPRESSIONVALUE>0</EXPRESSIONVALUE>
			                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
		                                            </FILTER>
	                                            </FILTERLIST>
	                                            <SELECTLIST>
		                                            <EXPRESSION>
			                                            <FIELDNAME>SECTIONID</FIELDNAME>
			                                            <ALIAS>SECTIONID</ALIAS>
		                                            </EXPRESSION>
		                                            <EXPRESSION>
			                                            <FIELDNAME>NAME</FIELDNAME>
			                                            <ALIAS>NAME</ALIAS>
		                                            </EXPRESSION>
		                                            <EXPRESSION>
			                                            <FIELDNAME>SORTORDER</FIELDNAME>
			                                            <ALIAS>SORTORDER</ALIAS>
		                                            </EXPRESSION>
		                                            <EXPRESSION>
			                                            <FIELDNAME>PATH</FIELDNAME>
			                                            <ALIAS>PATH</ALIAS>
		                                            </EXPRESSION>
		                                            <EXPRESSION>
			                                            <FIELDNAME>DESCRIPTION</FIELDNAME>
			                                            <ALIAS>DESCRIPTION</ALIAS>
		                                            </EXPRESSION>                                                    
	                                           </SELECTLIST>
                                            </RELATIONSHIP>
	                                        <SELECTLIST>
		                                        <EXPRESSION>
			                                        <FIELDNAME>SECTIONID</FIELDNAME>
			                                        <ALIAS>SECTIONID</ALIAS>
		                                        </EXPRESSION>
                                                <EXPRESSION>
		                                            <FIELDNAME>PATH</FIELDNAME>
		                                            <ALIAS>PATH</ALIAS>
	                                            </EXPRESSION>	
                                                <EXPRESSION>
		                                            <FIELDNAME>SORTORDER</FIELDNAME>
		                                            <ALIAS>SORTORDER</ALIAS>
	                                            </EXPRESSION>
	                                        </SELECTLIST>
                                        </TARGETENTITY>", sb_PathListToSearch.ToString());
                sb_SearchInput.Append("</SEARCH>");

            String str_NsearchInput = UpdateTreeConfigurationForSectionDisplay_NSearch(ref EmptyNode, sb_SearchInput.ToString(), "SubSection");
            ObjSearch       = new SearchConfiguration(SessionID, UserID, TenantName, CurrentLanguage);
            SearchError = ObjSearch.GetHierarchialData(str_NsearchInput, ref SearchResult);

            if (String.IsNullOrEmpty(SearchError) && !String.IsNullOrEmpty(SearchResult))
            {
                SubsectionList      = SearchResult;
                SearchDocument      = new XPathDocument(new System.IO.StringReader(SearchResult));
                xpNav_SearchRoot    = SearchDocument.CreateNavigator();

                for (int KeyCnt = 1; KeyCnt <= SectionPathList.Count; KeyCnt++)
                {
                    xpNav_Section = xpNav_SearchRoot.SelectSingleNode("SECTIONCOLLECTION/SECTION/PATH[translate(text(),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')=" + ALXmlUtil.GetXpathValueString(SectionPathList[KeyCnt].ToString().ToUpper()) + "]");
                    if (xpNav_Section != null)
                    {
                        xpNav_Section.MoveToParent();
                        ParentSectionIDList.Add(KeyCnt, xpNav_Section.SelectSingleNode("SECTIONID").Value);
                    }                    
                }                
            }
            #endregion

            return ParentSectionIDList;
        }

        private String GetDefaultSectionTree(String Message)
        {
            String DefaultSectionTreeXml = "";
            DefaultSectionTreeXml = "<TreeNodes>";
            DefaultSectionTreeXml = DefaultSectionTreeXml + "<TreeNode Selectionmode=\"None\" PostbackType=\"None\" NodeData=\"\" Text=\"" + Message + ".\">";
            DefaultSectionTreeXml = DefaultSectionTreeXml + "</TreeNode>";
            DefaultSectionTreeXml = DefaultSectionTreeXml + "</TreeNodes>";
            return DefaultSectionTreeXml;
        }

        private void PrepareBreadCrumbXML(String SelectedSectionID)
        {
            XPathDocument           xTreeDoc            = null;
            XPathNavigator          xTreeNavigator      = null;
            XPathNavigator          xCurrntTreeNav      = null;        
            StringBuilder           sb_BreadCrumbXml    = new StringBuilder();
            StringBuilder           sb_Tempxml          = new StringBuilder();
            StringBuilder           BreadCrumbFilePath  = new StringBuilder();
            String                  TreeXMLPath         = null;
            String                  str_SectionName     = null;
            String                  str_SectionDisplayName = null;
           
           
            TreeXMLPath     = ALSolutionCommon.GetTempPath(SessionID) + "DAM_SectionTreeInput.xml";
            if (File.Exists(TreeXMLPath))
            {
                xTreeDoc = new XPathDocument(TreeXMLPath);
                xTreeNavigator = xTreeDoc.CreateNavigator();
                xCurrntTreeNav = xTreeNavigator.SelectSingleNode("descendant::TreeNode[@NodeData='" + SelectedSectionID + "']");

                if (xCurrntTreeNav != null)
                {
                    str_SectionName = xCurrntTreeNav.GetAttribute("SECTIONNAME", "");
                    str_SectionDisplayName = str_SectionName;
                    if (str_SectionName.Length > 15)
                        str_SectionDisplayName = str_SectionName.Substring(0, 15) + "..";

                    sb_Tempxml.AppendFormat("<BreadCrumb Name=\"{0}\" DisplayText=\"{1}\" Selected=\"True\" RaiseEvent=\"True\" BreadcrumbTextMode=\"Text\" Tooltip=\"{2}\"> </BreadCrumb>", xCurrntTreeNav.GetAttribute("NodeData", ""), ALSolutionCommon.TrimAndReplaceSpecialChars(str_SectionDisplayName), ALSolutionCommon.TrimAndReplaceSpecialChars(str_SectionName));
                    while (xCurrntTreeNav.MoveToParent())
                    {
                        sb_BreadCrumbXml.Remove(0, sb_BreadCrumbXml.Length);
                        if (xCurrntTreeNav.GetAttribute("NodeData", "") == "Home")
                        {
                            sb_BreadCrumbXml.AppendFormat("<BreadCrumb Separator=\">\"><BreadCrumb Name=\"\" DisplayText=\"Home\" Selected=\"False\" RaiseEvent=\"False\" BreadcrumbTextMode=\"Text\" Tooltip=\"Home\">{0}</BreadCrumb></BreadCrumb>", sb_Tempxml);
                            break;
                        }
                        else
                        {
                            str_SectionName         = xCurrntTreeNav.GetAttribute("SECTIONNAME", "");
                            str_SectionDisplayName  = str_SectionName;
                            if (str_SectionName.Length > 15)
                                str_SectionDisplayName = str_SectionName.Substring(0, 15) + "..";

                            sb_BreadCrumbXml.AppendFormat("<BreadCrumb Name=\"{0}\" DisplayText=\"{1}\" Selected=\"False\" RaiseEvent=\"True\" BreadcrumbTextMode=\"Text\" Tooltip=\"{2}\">{3}</BreadCrumb>", xCurrntTreeNav.GetAttribute("NodeData", ""), ALSolutionCommon.TrimAndReplaceSpecialChars(str_SectionDisplayName), ALSolutionCommon.TrimAndReplaceSpecialChars(str_SectionName), sb_Tempxml);
                        }

                        sb_Tempxml.Remove(0, sb_Tempxml.Length);
                        sb_Tempxml.Append(sb_BreadCrumbXml);
                    }
                    if (String.IsNullOrEmpty(BreadCrumbControlInputFilePath))
                        BreadCrumbFilePath.AppendFormat("{0}Assetlist_BreadCrumbControlInput.xml", ALSolutionCommon.GetTempPath(SessionID));
                    else
                        BreadCrumbFilePath.Append(BreadCrumbControlInputFilePath);

                    File.WriteAllText(BreadCrumbFilePath.ToString(), sb_BreadCrumbXml.ToString());
                }
            }
        }

        /// <alsummary>
        /// <createddate> 19 Dec 2007 </createddate>
        /// <author> Mr. Pravin Shinde</author>
        /// <purpose> Prepare FileDownload page URL and Audit XML for current version of DigitalAsset.</purpose>
        /// </alsummary>
        public String PrepareDigitalAssetDownloadUrl(String DigitalAssetID, String RenditionType, String DownloadFileName, String AuditSolutionName, bool IsAuditXMLRequired,String RevisionID, ref String DownloadPageUrl, ref String AuditXML)
        {
            #region Variable declaration
            String          OutputXml                   = null;
            String          DigitlAssetName             = null;
            String          SectionPath                 = null;
            String          FileName                    = null;
            String          ObjectID                    = null;
            String          SchemaID                    = null;
            String          FileID                     = null;
            String          FieldName                   = null;
            String          FileFormat                  = null;
            String          FileSize                    = null;
            String          FilePath                    = null;
            String          IsAssetLocked               = null;
            String          ErrorXML                    = null;
            StringBuilder   sb_DownloadPageUrl          = new StringBuilder();
            StringBuilder   sb_SearchInputXml           = new StringBuilder();
            StringBuilder   sb_AuditInputXml            = new StringBuilder();
            XPathDocument   xpdoc_AssetRendition        = null;
            XPathNavigator  xpnav_RootAssetRendition    = null;
            XPathNavigator  xpnav_AssetRendition        = null;
            String          str_Relationshipname        = null;
            StringBuilder   sb_XPath                    = new StringBuilder();
            XPathNavigator  xpNav_AssetExpired          = null;
            XPathNavigator  xpNav_MarkedAsDeleted       = null;
            Boolean         IsFileOnMediaServer         = false;
            String          VaultName                   = null;
            String          ExpiryDate                  = null;
            String          VersionNumber               = null;
            String          VaultID                     = null;
            String          OriginalFileFormat          = null;
            String          RenditionID                 = null;
            String          OrgRenditionID              = null;
            #endregion

            try
            {
                #region Prepare Search Input XML
                sb_SearchInputXml.Append("<SEARCH>");
                sb_SearchInputXml.Append(GetSessionInfoNode());
                sb_SearchInputXml.Append("<TARGETOBJECT>DIGITALASSET</TARGETOBJECT>");
                sb_SearchInputXml.Append("<TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>");
                sb_SearchInputXml.Append("<AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>");
                sb_SearchInputXml.Append("<MAXRESULTSIZE>1</MAXRESULTSIZE>");
                sb_SearchInputXml.Append("<TARGETENTITY>");
                sb_SearchInputXml.Append("<PRIVILEGENAME>READ</PRIVILEGENAME>");
                sb_SearchInputXml.Append("<FILTERLIST>");
                sb_SearchInputXml.Append("<FILTER>");
                sb_SearchInputXml.Append("<FILTERID>1</FILTERID>");
                sb_SearchInputXml.Append("<EXPRESSION>DIGITALASSETID</EXPRESSION>");
                sb_SearchInputXml.Append("<OPERATOR>EQUALTO</OPERATOR>");
                sb_SearchInputXml.AppendFormat("<EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>", DigitalAssetID);
                sb_SearchInputXml.Append("<ISCASESENSITIVE>False</ISCASESENSITIVE>");
                sb_SearchInputXml.Append("</FILTER>");
                sb_SearchInputXml.Append("</FILTERLIST>");

                //For Original/preview files of current version get details of current versions renditions.
                if (String.IsNullOrEmpty(RevisionID))
                {
                    sb_SearchInputXml.Append("<RELATIONSHIP NAME=\"DIGITALASSET_CURRENTREVISION\">");
                    str_Relationshipname = "DIGITALASSET_CURRENTREVISION";
                }
                else
                {
                    sb_SearchInputXml.Append("<RELATIONSHIP NAME=\"DIGITALASSET_CONTENTREVISION\">");
                    sb_SearchInputXml.Append("<FILTERLIST>");
                    sb_SearchInputXml.Append("<FILTER>");
                    sb_SearchInputXml.Append("<FILTERID>1</FILTERID>");
                    sb_SearchInputXml.Append("<EXPRESSION>CONTENTREVISIONID</EXPRESSION>");
                    sb_SearchInputXml.Append("<OPERATOR>EQUALTO</OPERATOR>");
                    sb_SearchInputXml.AppendFormat("<EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>", RevisionID);
                    sb_SearchInputXml.Append("<ISCASESENSITIVE>False</ISCASESENSITIVE>");
                    sb_SearchInputXml.Append("</FILTER>");
                    sb_SearchInputXml.Append("</FILTERLIST>");
                    str_Relationshipname = "DIGITALASSET_CONTENTREVISION";
                }

                sb_SearchInputXml.Append("<SELECTLIST>");
                sb_SearchInputXml.Append("<EXPRESSION>");
                sb_SearchInputXml.Append("<FIELDNAME>CONTENTREVISIONID</FIELDNAME>");
                sb_SearchInputXml.Append("<ALIAS>CONTENTREVISIONID</ALIAS>");
                sb_SearchInputXml.Append("</EXPRESSION>");
                sb_SearchInputXml.Append("<EXPRESSION>");
                sb_SearchInputXml.Append("<FIELDNAME>VERSIONNUMBER</FIELDNAME>");
                sb_SearchInputXml.Append("<ALIAS>VERSIONNUMBER</ALIAS>");
                sb_SearchInputXml.Append("</EXPRESSION>");
                sb_SearchInputXml.Append("</SELECTLIST>");
                sb_SearchInputXml.Append("<RELATIONSHIP NAME=\"CONTENTREVISION_RENDITION\">");
                sb_SearchInputXml.Append("<FILTERLIST>");
                sb_SearchInputXml.Append("<FILTER>");
                sb_SearchInputXml.Append("<FILTERID>1</FILTERID>");
                sb_SearchInputXml.Append("<EXPRESSION>RENDITIONTYPE</EXPRESSION>");
                if (String.Compare(RenditionType, "Original", true) == 0)
                {
                    sb_SearchInputXml.Append("<OPERATOR>EQUALTO</OPERATOR>");
                    sb_SearchInputXml.AppendFormat("<EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>", RenditionType);
                }
                else
                {
                    sb_SearchInputXml.Append("<OPERATOR>IN</OPERATOR>");
                    sb_SearchInputXml.AppendFormat("<EXPRESSIONVALUE>('{0}', 'Original')</EXPRESSIONVALUE>", RenditionType);
                }
                sb_SearchInputXml.Append("<ISCASESENSITIVE>False</ISCASESENSITIVE>");
                sb_SearchInputXml.Append("</FILTER>");
                sb_SearchInputXml.Append("</FILTERLIST>");
                sb_SearchInputXml.Append("<SELECTLIST>");
                sb_SearchInputXml.Append("<EXPRESSION>");
                sb_SearchInputXml.Append("<FIELDNAME>ASSETRENDITIONID</FIELDNAME>");
                sb_SearchInputXml.Append("<ALIAS>ASSETRENDITIONID</ALIAS>");
                sb_SearchInputXml.Append("</EXPRESSION>");
                sb_SearchInputXml.Append("<EXPRESSION>");
                sb_SearchInputXml.Append("<FIELDNAME>NAME</FIELDNAME>");
                sb_SearchInputXml.Append("<ALIAS>NAME</ALIAS>");
                sb_SearchInputXml.Append("</EXPRESSION>");
                sb_SearchInputXml.Append("<EXPRESSION>");
                sb_SearchInputXml.Append("<FIELDNAME>FILENAME</FIELDNAME>");
                sb_SearchInputXml.Append("<ALIAS>FILENAME</ALIAS>");
                sb_SearchInputXml.Append("</EXPRESSION>");
                sb_SearchInputXml.Append("<EXPRESSION>");
                sb_SearchInputXml.Append("<FIELDNAME>ACTUALFILE</FIELDNAME>");
                sb_SearchInputXml.Append("<ALIAS>ACTUALFILE</ALIAS>");
                sb_SearchInputXml.Append("</EXPRESSION>");
                sb_SearchInputXml.Append("<EXPRESSION>");
                sb_SearchInputXml.Append("<FIELDNAME>FILEFORMAT</FIELDNAME>");
                sb_SearchInputXml.Append("<ALIAS>FILEFORMAT</ALIAS>");
                sb_SearchInputXml.Append("</EXPRESSION>");
                sb_SearchInputXml.Append("<EXPRESSION>");
                sb_SearchInputXml.Append("<FIELDNAME>FILESIZE</FIELDNAME>");
                sb_SearchInputXml.Append("<ALIAS>FILESIZE</ALIAS>");
                sb_SearchInputXml.Append("</EXPRESSION>");
                sb_SearchInputXml.Append("<EXPRESSION>");
                sb_SearchInputXml.Append("<FIELDNAME>RENDITIONTYPE</FIELDNAME>");
                sb_SearchInputXml.Append("<ALIAS>RENDITIONTYPE</ALIAS>");
                sb_SearchInputXml.Append("</EXPRESSION>");
                sb_SearchInputXml.Append("</SELECTLIST>");
                sb_SearchInputXml.Append("</RELATIONSHIP>");
                sb_SearchInputXml.Append("</RELATIONSHIP>");
                sb_SearchInputXml.Append("<RELATIONSHIP NAME=\"SECTION_DIGITALASSET\">");
                sb_SearchInputXml.Append("<SELECTLIST>");
                sb_SearchInputXml.Append("<EXPRESSION>");
                sb_SearchInputXml.Append("<FIELDNAME>NAME</FIELDNAME>");
                sb_SearchInputXml.Append("<ALIAS>NAME</ALIAS>");
                sb_SearchInputXml.Append("</EXPRESSION>");
                sb_SearchInputXml.Append("<EXPRESSION>");
                sb_SearchInputXml.Append("<FIELDNAME>PATH</FIELDNAME>");
                sb_SearchInputXml.Append("<ALIAS>PATH</ALIAS>");
                sb_SearchInputXml.Append("</EXPRESSION>");
                sb_SearchInputXml.Append("</SELECTLIST>");
                sb_SearchInputXml.Append(@"<RELATIONSHIP NAME='SECTION_VAULT'>
	                                        <SELECTLIST>
		                                        <EXPRESSION>
			                                        <FIELDNAME>VAULTID</FIELDNAME>
			                                        <ALIAS>VAULTID</ALIAS>
		                                        </EXPRESSION>
                                                <EXPRESSION>
			                                        <FIELDNAME>NAME</FIELDNAME>
			                                        <ALIAS>NAME</ALIAS>
		                                        </EXPRESSION>
	                                        </SELECTLIST>
			                               </RELATIONSHIP>");
                sb_SearchInputXml.Append("</RELATIONSHIP>");
                sb_SearchInputXml.Append("<SELECTLIST>");
                sb_SearchInputXml.Append("<EXPRESSION>");
                sb_SearchInputXml.Append("<FIELDNAME>DIGITALASSETID</FIELDNAME>");
                sb_SearchInputXml.Append("<ALIAS>DIGITALASSETID</ALIAS>");
                sb_SearchInputXml.Append("</EXPRESSION>");
                sb_SearchInputXml.Append("<EXPRESSION>");
                sb_SearchInputXml.Append("<FIELDNAME>NAME</FIELDNAME>");
                sb_SearchInputXml.Append("<ALIAS>NAME</ALIAS>");
                sb_SearchInputXml.Append("</EXPRESSION>");
                sb_SearchInputXml.Append("<EXPRESSION>");
                sb_SearchInputXml.Append("<FIELDNAME>ISLOCKED</FIELDNAME>");
                sb_SearchInputXml.Append("<ALIAS>ISLOCKED</ALIAS>");
                sb_SearchInputXml.Append("</EXPRESSION>");
                sb_SearchInputXml.Append("<EXPRESSION>");
                sb_SearchInputXml.Append("<FIELDNAME>ASSETEXPIRED</FIELDNAME>");
                sb_SearchInputXml.Append("<ALIAS>ASSETEXPIRED</ALIAS>");
                sb_SearchInputXml.Append("</EXPRESSION>");
                sb_SearchInputXml.Append("<EXPRESSION>");
                sb_SearchInputXml.Append("<FIELDNAME>MARKEDASDELETED</FIELDNAME>");
                sb_SearchInputXml.Append("<ALIAS>MARKEDASDELETED</ALIAS>");
                sb_SearchInputXml.Append("</EXPRESSION>");
                sb_SearchInputXml.Append(@"<EXPRESSION>
				                             <FIELDNAME>EXPIRYDATE</FIELDNAME>
				                             <ALIAS>EXPIRYDATE</ALIAS>
			                               </EXPRESSION>");
                sb_SearchInputXml.Append("</SELECTLIST>");

                sb_SearchInputXml.Append("</TARGETENTITY>");
                sb_SearchInputXml.Append("</SEARCH>");

                #endregion

                #region Do Search
                ObjSearchConfiguration  = new SearchConfiguration(SessionID, UserID, TenantName, CurrentLanguage);
                ErrorXML                = ObjSearchConfiguration.GetHierarchialData(sb_SearchInputXml.ToString(), ref OutputXml);
                #endregion

                if (!String.IsNullOrEmpty(OutputXml))
                {                    
                    xpdoc_AssetRendition        = new XPathDocument(new StringReader(OutputXml));
                    xpnav_RootAssetRendition    = xpdoc_AssetRendition.CreateNavigator();
                    
                    //Check if Asset is marked for delete or permanat delete.    
                    xpNav_MarkedAsDeleted = xpnav_RootAssetRendition.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/MARKEDASDELETED");
                    if (xpNav_MarkedAsDeleted != null && xpNav_MarkedAsDeleted.Value != "0")
                    {
                        switch (xpNav_MarkedAsDeleted.Value)
                        {
                            case "1":
                                ErrorXML = "DIGITALASSET_DIGITALASSETMARKEDASDELETED_Shared";
                                break;
                            case "2":
                                ErrorXML = "DIGITALASSET_DIGITALASSETMARKEDFORPERMANANTDELETE_Shared";
                                break;
                        }
                        return ErrorXML;
                    }

                    //Check if Download of Expired Asset is allowed.
                    if (!AllowDownloadOfExpiredAsset)
                    {
                        xpNav_AssetExpired = xpnav_RootAssetRendition.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/ASSETEXPIRED");
                        if(xpNav_AssetExpired != null && xpNav_AssetExpired.Value == "1")
                            return "DAMFACADE_DAMHELPER_DOWNLOADDIGITALASSET_DIGITALASSETEXPIRED";                        
                    }

                    if (String.Compare(RenditionType, "ORIGINAL", true) == 0)
                    {
                        //Is Asset Locked 
                        xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/ISLOCKED");
                        if (xpnav_AssetRendition != null)
                            IsAssetLocked = xpnav_AssetRendition.Value;
                        else
                            IsAssetLocked = "";

                        if (String.Compare(IsAssetLocked, "1") == 0)                     
                            return "DAMFACADE_DAMHELPER_DOWNLOADDIGITALASSET_DIGITALASSETLOCKED";                        
                    }
                    sb_XPath.AppendFormat("DIGITALASSETCOLLECTION/DIGITALASSET/{0}/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION[RENDITIONTYPE/text()='{1}']/ASSETRENDITIONID", str_Relationshipname, RenditionType);
                    xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                    if (xpnav_AssetRendition != null)
                    {
                        RenditionID = xpnav_AssetRendition.Value;

                        //Get Original Rendition ID of that version
                        sb_XPath.Remove(0, sb_XPath.Length);
                        sb_XPath.AppendFormat("DIGITALASSETCOLLECTION/DIGITALASSET/{0}/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION[RENDITIONTYPE/text()='Original']/ASSETRENDITIONID", str_Relationshipname, RenditionType);
                        xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                        if (xpnav_AssetRendition != null)
                            OrgRenditionID = xpnav_AssetRendition.Value;

                        #region Get Parameters
                        //DigitalAsset Name
                        xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/NAME");
                        if (xpnav_AssetRendition != null)
                            DigitlAssetName = xpnav_AssetRendition.Value;

                        //Field Name 
                        sb_XPath.Remove(0, sb_XPath.Length);
                        sb_XPath.AppendFormat("DIGITALASSETCOLLECTION/DIGITALASSET/{0}/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION[ASSETRENDITIONID='{1}']/FILESYSTEM/FILE/FIELDNAME", str_Relationshipname, RenditionID);
                        xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                        if (xpnav_AssetRendition != null)
                            FieldName = xpnav_AssetRendition.Value;

                        //Object ID
                        sb_XPath.Remove(0, sb_XPath.Length);
                        sb_XPath.AppendFormat("DIGITALASSETCOLLECTION/DIGITALASSET/{0}/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION[ASSETRENDITIONID='{1}']/FILESYSTEM/FILE/OBJECTID", str_Relationshipname, RenditionID);
                        xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                        if (xpnav_AssetRendition != null)
                            ObjectID = xpnav_AssetRendition.Value;

                        //Schema ID
                        sb_XPath.Remove(0, sb_XPath.Length);
                        sb_XPath.AppendFormat("DIGITALASSETCOLLECTION/DIGITALASSET/{0}/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION[ASSETRENDITIONID='{1}']/FILESYSTEM/FILE/SCHEMAID", str_Relationshipname, RenditionID);
                        xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                        if (xpnav_AssetRendition != null)
                            SchemaID = xpnav_AssetRendition.Value;

                        //File Name
                        sb_XPath.Remove(0, sb_XPath.Length);
                        sb_XPath.AppendFormat("DIGITALASSETCOLLECTION/DIGITALASSET/{0}/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION[ASSETRENDITIONID='{1}']/FILESYSTEM/FILE/NAME", str_Relationshipname, RenditionID);
                        xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                        if (xpnav_AssetRendition != null)
                            FileName = xpnav_AssetRendition.Value;

                        //File Format / File Extension
                        sb_XPath.Remove(0, sb_XPath.Length);
                        sb_XPath.AppendFormat("DIGITALASSETCOLLECTION/DIGITALASSET/{0}/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION[ASSETRENDITIONID='{2}' and RENDITIONTYPE/text()='{1}']/FILEFORMAT", str_Relationshipname, RenditionType, RenditionID);
                        xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                        if (xpnav_AssetRendition != null)
                            FileFormat = xpnav_AssetRendition.Value;

                        //Original File Format / File Extension
                        sb_XPath.Remove(0, sb_XPath.Length);
                        sb_XPath.AppendFormat("DIGITALASSETCOLLECTION/DIGITALASSET/{0}/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION[ASSETRENDITIONID='{1}' and RENDITIONTYPE/text()='Original']/FILEFORMAT", str_Relationshipname, OrgRenditionID);
                        xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                        if (xpnav_AssetRendition != null)
                            OriginalFileFormat = xpnav_AssetRendition.Value;
                        
                        //File Size
                        sb_XPath.Remove(0, sb_XPath.Length);
                        sb_XPath.AppendFormat("DIGITALASSETCOLLECTION/DIGITALASSET/{0}/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION[ASSETRENDITIONID='{1}']/FILESIZE", str_Relationshipname, RenditionID);
                        xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                        if (xpnav_AssetRendition != null)
                            FileSize = xpnav_AssetRendition.Value;

                        //IsFileOnMediaServer
                        sb_XPath.Remove(0, sb_XPath.Length);
                        sb_XPath.AppendFormat("DIGITALASSETCOLLECTION/DIGITALASSET/{0}/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION[ASSETRENDITIONID='{1}']/FILESYSTEM/FILE/ONMEDIASERVER", str_Relationshipname, RenditionID);
                        xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                        if (xpnav_AssetRendition != null)
                            IsFileOnMediaServer = Boolean.Parse(xpnav_AssetRendition.Value);

                        if (IsFileOnMediaServer)
                        {
                            //FileID
                            sb_XPath.Remove(0, sb_XPath.Length);
                            sb_XPath.AppendFormat("DIGITALASSETCOLLECTION/DIGITALASSET/{0}/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION[ASSETRENDITIONID='{1}']/FILESYSTEM/FILE/FILEID", str_Relationshipname, RenditionID);
                            xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                            if (xpnav_AssetRendition != null)
                                FileID = xpnav_AssetRendition.Value;

                            //FilePath
                            sb_XPath.Remove(0, sb_XPath.Length);
                            sb_XPath.AppendFormat("DIGITALASSETCOLLECTION/DIGITALASSET/{0}/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION[ASSETRENDITIONID='{1}']/FILESYSTEM/FILE/PATH", str_Relationshipname, RenditionID);
                            xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                            if (xpnav_AssetRendition != null)
                                FilePath = xpnav_AssetRendition.Value;
                        }


                        //Sectio Path
                        xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/SECTION_DIGITALASSET/SECTION/PATH");
                        if (xpnav_AssetRendition != null)
                            SectionPath = xpnav_AssetRendition.Value;

                        //VaultName
                        xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/SECTION_DIGITALASSET/SECTION/SECTION_VAULT/VAULT/NAME");
                        if (xpnav_AssetRendition != null)
                            VaultName = xpnav_AssetRendition.Value;

                        //VaultID
                        xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/SECTION_DIGITALASSET/SECTION/SECTION_VAULT/VAULT/VAULTID");
                        if (xpnav_AssetRendition != null)
                            VaultID = xpnav_AssetRendition.Value;

                        //Expiry Date
                        xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/EXPIRYDATE");
                        if (xpnav_AssetRendition != null)
                            ExpiryDate = xpnav_AssetRendition.InnerXml;

                        //Version Number
                        sb_XPath.Remove(0, sb_XPath.Length);
                        sb_XPath.AppendFormat("DIGITALASSETCOLLECTION/DIGITALASSET/{0}/CONTENTREVISION/VERSIONNUMBER", str_Relationshipname);
                        xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                        if (xpnav_AssetRendition != null)
                            VersionNumber = xpnav_AssetRendition.Value;
                        #endregion

                        #region Prepare FileDownload Page URL
                        sb_DownloadPageUrl.Append(ALSolutionCommon.PrepareFileDownloadURL(IsFileOnMediaServer, TenantName));
                        sb_DownloadPageUrl.AppendFormat("TenantName={0}&Language={1}&UserID={2}&SessionID={3}&ThemeName={4}&IsFileOnMediaServer={5}", TenantName, CurrentLanguage, UserID, SessionID, ThemeName, IsFileOnMediaServer.ToString());

                        if (!String.IsNullOrEmpty(FileID))
                            sb_DownloadPageUrl.AppendFormat("&FileID={0}", FileID);

                        if (!String.IsNullOrEmpty(FieldName))
                            sb_DownloadPageUrl.AppendFormat("&FieldName={0}", FieldName);

                        if (!String.IsNullOrEmpty(ObjectID))
                            sb_DownloadPageUrl.AppendFormat("&ObjectID={0}", ObjectID);

                        if (!String.IsNullOrEmpty(SchemaID))
                            sb_DownloadPageUrl.AppendFormat("&SchemaID={0}", SchemaID);

                        if (!String.IsNullOrEmpty(FilePath))
                            sb_DownloadPageUrl.AppendFormat("&FilePath={0}", System.Web.HttpUtility.UrlEncode(FilePath));

                        if (!String.IsNullOrEmpty(FileName))
                            sb_DownloadPageUrl.AppendFormat("&FileName={0}", System.Web.HttpUtility.UrlEncode(FileName));

                        if (!String.IsNullOrEmpty(FileFormat))
                            sb_DownloadPageUrl.AppendFormat("&FileExtension={0}", FileFormat);

                        if (String.IsNullOrEmpty(DownloadFileName))
                            DownloadFileName = DigitlAssetName;

                        CheckAssetName(ref DigitlAssetName, OriginalFileFormat);
                        DownloadFileName = GetDownloadFileName(FileFormat, DigitlAssetName, FileName, RenditionType);
                        switch (RenditionType)
                        {
                            case "Original":
                                break;
                            case "Preview":
                                sb_DownloadPageUrl.Append("&ViewPreview=YES");                              
                                break;
                        }

                        if (!String.IsNullOrEmpty(DownloadFileName))
                            sb_DownloadPageUrl.AppendFormat("&DownloadFileName={0}", System.Web.HttpUtility.UrlEncode(DownloadFileName));
                        #endregion

                        #region Prepare Audit XML

                        if (IsAuditXMLRequired)
                        {
                            DigitlAssetName = ALSolutionCommon.TrimAndReplaceSpecialChars(DigitlAssetName);
                            SectionPath = ALSolutionCommon.TrimAndReplaceSpecialChars(SectionPath);
                            FileName = ALSolutionCommon.TrimAndReplaceSpecialChars(FileName);
                            VaultName = ALSolutionCommon.TrimAndReplaceSpecialChars(VaultName);
                            ExpiryDate = ALSolutionCommon.TrimAndReplaceSpecialChars(ExpiryDate);

                            sb_AuditInputXml.Append("<AuditParams>");
                            sb_AuditInputXml.Append("<Param Name='AUDIT' Value='true'/>");
                            sb_AuditInputXml.Append("<Param Name='AssetDownloaded' Value='true'/>");
                            sb_AuditInputXml.AppendFormat("<Param Name='ENTITY_ID' Value='{0}'/>", DigitalAssetID);
                            sb_AuditInputXml.AppendFormat("<Param Name='LOGDESCRIPTION' Value='Downloaded the {0} file of {1} from section {2}.'/>", RenditionType, DigitlAssetName, SectionPath);
                            sb_AuditInputXml.AppendFormat("<Param Name='ENTITY_NAME' Value='{0}'/>", DigitlAssetName);
                            sb_AuditInputXml.AppendFormat("<Param Name='SECTIONPATH' Value='{0}'/>", SectionPath);
                            sb_AuditInputXml.AppendFormat("<Param Name='VAULTNAME' Value='{0}'/>", VaultName);
                            sb_AuditInputXml.AppendFormat("<Param Name='EXPIRYDATE' Value='{0}'/>", ExpiryDate);
                            sb_AuditInputXml.AppendFormat("<Param Name='DOWNLOADEDVERSIONNUMBER' Value='{0}'/>", VersionNumber);
                            sb_AuditInputXml.AppendFormat("<Param Name='VAULTID' Value='{0}'/>", VaultID);

                            if (!String.IsNullOrEmpty(AuditSolutionName) && !AuditSolutionName.Equals("DIGITAL LIBRARY"))
                                sb_AuditInputXml.AppendFormat("<Param Name='APPNAME' Value='{0}'/>", AuditSolutionName);
                            else
                                sb_AuditInputXml.Append("<Param Name='APPNAME' Value='DIGITAL LIBRARY'/>");

                            if (!String.IsNullOrEmpty(AuditSolutionName) && AuditSolutionName.Equals("SITEBUILDER"))
                            {
                                sb_AuditInputXml.AppendFormat("<Param Name='ASSETNAME' Value='{0}'/>", DigitlAssetName);
                                sb_AuditInputXml.AppendFormat("<Param Name='LOCATION'  Value='{0}'/>", SectionPath);
                                sb_AuditInputXml.AppendFormat("<Param Name='FILETYPE' Value='{0}'/>", FileFormat);
                                sb_AuditInputXml.AppendFormat("<Param Name='RENDITIONTYPE' Value='{0}'/>", RenditionType);
                                sb_AuditInputXml.AppendFormat("<Param Name='FILESIZE' Value='{0}'/>", FileSize);
                                sb_AuditInputXml.AppendFormat("<Param Name='FILENAME' Value='{0}'/>", FileName);
                            }

                            switch (RenditionType)
                            {
                                case "Original":
                                    sb_AuditInputXml.Append("<Param Name ='ACTION' Value = 'DOWNLOAD DIGITALASSET' />");
                                    
                                    break;
                                case "Preview":
                                    sb_AuditInputXml.Append("<Param Name ='ACTION' Value = 'VIEW PREVIEW' />");
                                    break;
                            }
                            sb_AuditInputXml.Append("</AuditParams>");
                            AuditXML = sb_AuditInputXml.ToString();
                        }
                        #endregion

                        ALSolutionCommonObj = new ALSolutionCommon();
                        DownloadPageUrl = ALSolutionCommonObj.EncodeURLForResponseRedirects(sb_DownloadPageUrl.ToString());
                    }
                    else
                        return "DAMFACADE_DAMHELPER_DOWNLOADDIGITALASSET_RENDITIONNOTAVAILABLE";  
                }
            }
            catch (Exception ex)
            {
                ErrorXML = ALSolutionCommon.PrepareErrorXML(ex.Message);
                return ErrorXML;
            }
            return ErrorXML;
        }


        /// <alsummary>
        /// <createddate>26 Aug 2008</createddate>
        /// <author> Mr.Pravin Shinde</author>
        /// <purpose> Prepare FileDownload page URL and Audit XML for current version of DigitalAsset.
        /// This Method accepts FileSystem search Output XML navigator</purpose>
        /// </alsummary>
        public ERRORXML PrepareDigitalAssetDownloadUrl(XPathNavigator xpnav_RootAssetRendition, String str_Xpath, String DigitalAssetID, String RenditionType, String DownloadFileName, String AuditSolutionName, bool IsAuditXMLRequired, String RevisionID, ref String DownloadPageUrl, ref String AuditXML)
        {
            String				str_DigitlAssetName		= null;
            String				str_FieldName			= null;
            String				str_ObjectID			= null;
            String				str_SchemaID			= null;
            String              str_FileID              = null;
            String				str_FileName			= null;
            String              str_FilePath            = null;
            String				str_FileFormat			= null;
            String				str_FileSize			= null;
            String				SectionPath				= null;
            XPathNavigator		xpnav_AssetRendition	= null;
            XPathNavigator      xpNav_MarkedAsDeleted   = null;
            XPathNavigator      xpNav_AssetExpired      = null;
            StringBuilder		sb_DownloadPageUrl		= null;
            StringBuilder		sb_XPath				= null;
            StringBuilder		sb_AuditInputXml		= null;
            String				IsAssetLocked			= null;
            ALErrorCommon		obj_ALErrorCommon		= null;
            Boolean             IsFileOnMediaServer     = false;
            String              VaultName               = null;
            String              ExpiryDate              = null;
            String              VersionNumber           = null;
            String              VaultID                 = null;            
            
            try
            {
                sb_DownloadPageUrl = new StringBuilder(100);
                sb_XPath = new StringBuilder(100);
                obj_ALErrorCommon = new ALErrorCommon();

                if (xpnav_RootAssetRendition != null)
                {
                    //Check if Asset is marked for delete or permanat delete.    
                    xpNav_MarkedAsDeleted = xpnav_RootAssetRendition.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/MARKEDASDELETED");
                    if (xpNav_MarkedAsDeleted != null && xpNav_MarkedAsDeleted.Value != "0")
                    {
                        switch (xpNav_MarkedAsDeleted.Value)
                        {
                            case "1":
                                ErrorXML = "DIGITALASSET_DIGITALASSETMARKEDASDELETED_Shared";
                                break;
                            case "2":
                                ErrorXML = "DIGITALASSET_DIGITALASSETMARKEDFORPERMANANTDELETE_Shared";
                                break;
                        }
                        return ErrorXML;
                    }

                    //Check if Download of Expired Asset is allowed.
                    if (!AllowDownloadOfExpiredAsset)
                    {
                        xpNav_AssetExpired = xpnav_RootAssetRendition.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/ASSETEXPIRED");
                        if (xpNav_AssetExpired != null && xpNav_AssetExpired.Value == "1")
                            return "DAMFACADE_DAMHELPER_DOWNLOADDIGITALASSET_DIGITALASSETEXPIRED";
                    }

                    if (String.Compare(RenditionType, "ORIGINAL", true) == 0)
                    {
                        //Is Asset Locked 
                        xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/ISLOCKED");
                        if (xpnav_AssetRendition != null)
                            IsAssetLocked = xpnav_AssetRendition.Value;
                        else
                            IsAssetLocked = "";

                        if (String.Compare(IsAssetLocked, "1") == 0)
                            return "DAMFACADE_DAMHELPER_DOWNLOADDIGITALASSET_DIGITALASSETLOCKED";
                    }

                    #region Get Parameters
                    //DigitalAsset Name
                    xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/NAME");
                    if (xpnav_AssetRendition != null)
                        str_DigitlAssetName = xpnav_AssetRendition.Value;

                    //Field Name 
                    sb_XPath.Remove(0, sb_XPath.Length);
                    sb_XPath.AppendFormat("{0}/FILESYSTEM/FILE/FIELDNAME", str_Xpath);
                    xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                    if (xpnav_AssetRendition != null)
                        str_FieldName = xpnav_AssetRendition.Value;

                    //Object ID
                    sb_XPath.Remove(0, sb_XPath.Length);
                    sb_XPath.AppendFormat("{0}/FILESYSTEM/FILE/OBJECTID", str_Xpath);
                    xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                    if (xpnav_AssetRendition != null)
                        str_ObjectID = xpnav_AssetRendition.Value;

                    //Schema ID
                    sb_XPath.Remove(0, sb_XPath.Length);
                    sb_XPath.AppendFormat("{0}/FILESYSTEM/FILE/SCHEMAID", str_Xpath);
                    xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                    if (xpnav_AssetRendition != null)
                        str_SchemaID = xpnav_AssetRendition.Value;

                    //File Name
                    sb_XPath.Remove(0, sb_XPath.Length);
                    sb_XPath.AppendFormat("{0}/FILESYSTEM/FILE/NAME", str_Xpath);
                    xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                    if (xpnav_AssetRendition != null)
                        str_FileName = xpnav_AssetRendition.Value;

                    //File Format / File Extension
                    sb_XPath.Remove(0, sb_XPath.Length);
                    sb_XPath.AppendFormat("{0}/FILEFORMAT", str_Xpath);
                    xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                    if (xpnav_AssetRendition != null)
                        str_FileFormat = xpnav_AssetRendition.Value;

                    //File Size
                    sb_XPath.Remove(0, sb_XPath.Length);
                    sb_XPath.AppendFormat("{0}/FILESIZE", str_Xpath);
                    xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                    if (xpnav_AssetRendition != null)
                        str_FileSize = xpnav_AssetRendition.Value;

                    //IsFileOnMediaServer
                    sb_XPath.Remove(0, sb_XPath.Length);
                    sb_XPath.AppendFormat("{0}/FILESYSTEM/FILE/ONMEDIASERVER", str_Xpath);
                    xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                    if (xpnav_AssetRendition != null)
                        IsFileOnMediaServer = Boolean.Parse(xpnav_AssetRendition.Value);

                    if (IsFileOnMediaServer)
                    {
                        //FileID
                        sb_XPath.Remove(0, sb_XPath.Length);
                        sb_XPath.AppendFormat("{0}/FILESYSTEM/FILE/FILEID", str_Xpath);
                        xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                        if (xpnav_AssetRendition != null)
                            str_FileID = xpnav_AssetRendition.Value;

                        //FilePath
                        sb_XPath.Remove(0, sb_XPath.Length);
                        sb_XPath.AppendFormat("{0}/FILESYSTEM/FILE/PATH", str_Xpath);
                        xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                        if (xpnav_AssetRendition != null)
                            str_FilePath = xpnav_AssetRendition.Value;
                    }

                    //Section Path
                    xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/SECTION_DIGITALASSET/SECTION/PATH");
                    if (xpnav_AssetRendition != null)
                        SectionPath = xpnav_AssetRendition.Value;

                    //VaultName
                    xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/SECTION_DIGITALASSET/SECTION/SECTION_VAULT/VAULT/NAME");
                    if (xpnav_AssetRendition != null)
                        VaultName = xpnav_AssetRendition.Value;

                    //VaultID
                    xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/SECTION_DIGITALASSET/SECTION/SECTION_VAULT/VAULT/VAULTID");
                    if (xpnav_AssetRendition != null)
                        VaultID = xpnav_AssetRendition.Value;

                    //Expiry Date
                    xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/EXPIRYDATE");
                    if (xpnav_AssetRendition != null)
                        ExpiryDate = xpnav_AssetRendition.InnerXml;

                    //Version Number
                    sb_XPath.Remove(0, sb_XPath.Length);
                    sb_XPath.AppendFormat("DIGITALASSETCOLLECTION/DIGITALASSET/*/CONTENTREVISION/VERSIONNUMBER");
                    xpnav_AssetRendition = xpnav_RootAssetRendition.SelectSingleNode(sb_XPath.ToString());
                    if (xpnav_AssetRendition != null)
                        VersionNumber = xpnav_AssetRendition.Value;
                    #endregion

                    #region Prepare FileDownload Page URL

                    sb_DownloadPageUrl.Append(ALSolutionCommon.PrepareFileDownloadURL(IsFileOnMediaServer, TenantName));
                    sb_DownloadPageUrl.AppendFormat("TenantName={0}&Language={1}&UserID={2}&SessionID={3}&ThemeName={4}&IsFileOnMediaServer={5}", TenantName, CurrentLanguage, UserID, SessionID, ThemeName, IsFileOnMediaServer.ToString());

                    if (!String.IsNullOrEmpty(str_FileID))
                        sb_DownloadPageUrl.AppendFormat("&FileID={0}", str_FileID);

                    if (!String.IsNullOrEmpty(str_FieldName))
                        sb_DownloadPageUrl.AppendFormat("&FieldName={0}", str_FieldName);

                    if (!String.IsNullOrEmpty(str_ObjectID))
                        sb_DownloadPageUrl.AppendFormat("&ObjectID={0}", str_ObjectID);

                    if (!String.IsNullOrEmpty(str_SchemaID))
                        sb_DownloadPageUrl.AppendFormat("&SchemaID={0}", str_SchemaID);

                    if (!String.IsNullOrEmpty(str_FilePath))
                        sb_DownloadPageUrl.AppendFormat("&FilePath={0}", System.Web.HttpUtility.UrlEncode(str_FilePath));

                    if (!String.IsNullOrEmpty(str_FileName))
                        sb_DownloadPageUrl.AppendFormat("&FileName={0}", System.Web.HttpUtility.UrlEncode(str_FileName));

                    if (!String.IsNullOrEmpty(str_FileFormat))
                        sb_DownloadPageUrl.AppendFormat("&FileExtension={0}", str_FileFormat);

                    if (String.IsNullOrEmpty(DownloadFileName))
                        DownloadFileName = str_DigitlAssetName;

                    CheckAssetName(ref str_DigitlAssetName, str_FileFormat);
                    DownloadFileName = GetDownloadFileName(str_FileFormat, str_DigitlAssetName, str_FileName, RenditionType);
                    switch (RenditionType)
                    {
                        case "Original":
                            break;
                        case "Preview":
                            sb_DownloadPageUrl.Append("&ViewPreview=YES");
                            break;
                    }

                    if (!String.IsNullOrEmpty(DownloadFileName))
                        sb_DownloadPageUrl.AppendFormat("&DownloadFileName={0}", System.Web.HttpUtility.UrlEncode(DownloadFileName));
                    #endregion

                    #region Prepare Audit XML

                    if (IsAuditXMLRequired)
                    {
                        sb_AuditInputXml = new StringBuilder(100);
                        str_DigitlAssetName = ALSolutionCommon.TrimAndReplaceSpecialChars(str_DigitlAssetName);
                        SectionPath = ALSolutionCommon.TrimAndReplaceSpecialChars(SectionPath);
                        str_FileName = ALSolutionCommon.TrimAndReplaceSpecialChars(str_FileName);
                        VaultName = ALSolutionCommon.TrimAndReplaceSpecialChars(VaultName);
                        ExpiryDate = ALSolutionCommon.TrimAndReplaceSpecialChars(ExpiryDate);

                        sb_AuditInputXml.Append("<AuditParams>");
                        sb_AuditInputXml.Append("<Param Name='AUDIT' Value='true'/>");
                        sb_AuditInputXml.Append("<Param Name='AssetDownloaded' Value='true'/>");
                        sb_AuditInputXml.AppendFormat("<Param Name='ENTITY_ID' Value='{0}'/>", DigitalAssetID);
                        sb_AuditInputXml.AppendFormat("<Param Name='LOGDESCRIPTION' Value='Downloaded the {0} file of {1} from section {2}.'/>", RenditionType, str_DigitlAssetName, SectionPath);
                        sb_AuditInputXml.AppendFormat("<Param Name='ENTITY_NAME' Value='{0}'/>", str_DigitlAssetName);
                        sb_AuditInputXml.AppendFormat("<Param Name='SECTIONPATH' Value='{0}'/>", SectionPath);
                        sb_AuditInputXml.AppendFormat("<Param Name='VAULTNAME' Value='{0}'/>", VaultName);
                        sb_AuditInputXml.AppendFormat("<Param Name='EXPIRYDATE' Value='{0}'/>", ExpiryDate);
                        sb_AuditInputXml.AppendFormat("<Param Name='VAULTID' Value='{0}'/>", VaultID);
                        sb_AuditInputXml.AppendFormat("<Param Name='DOWNLOADEDVERSIONNUMBER' Value='{0}'/>", VersionNumber);


                        if (!String.IsNullOrEmpty(AuditSolutionName) && !AuditSolutionName.Equals("DIGITAL LIBRARY"))
                            sb_AuditInputXml.AppendFormat("<Param Name='APPNAME' Value='{0}'/>", AuditSolutionName);
                        else
                            sb_AuditInputXml.Append("<Param Name='APPNAME' Value='DIGITAL LIBRARY'/>");

                        if (!String.IsNullOrEmpty(AuditSolutionName) && AuditSolutionName.Equals("SITEBUILDER"))
                        {
                            sb_AuditInputXml.AppendFormat("<Param Name='ASSETNAME' Value='{0}'/>", str_DigitlAssetName);
                            sb_AuditInputXml.AppendFormat("<Param Name='LOCATION'  Value='{0}'/>", SectionPath);
                            sb_AuditInputXml.AppendFormat("<Param Name='FILETYPE' Value='{0}'/>", str_FileFormat);
                            sb_AuditInputXml.AppendFormat("<Param Name='RENDITIONTYPE' Value='{0}'/>", RenditionType);
                            sb_AuditInputXml.AppendFormat("<Param Name='FILESIZE' Value='{0}'/>", str_FileSize);
                            sb_AuditInputXml.AppendFormat("<Param Name='FILENAME' Value='{0}'/>", str_FileName);
                        }

                        switch (RenditionType)
                        {
                            case "Original":
                                sb_AuditInputXml.Append("<Param Name ='ACTION' Value = 'DOWNLOAD DIGITALASSET' />");
                               
                                break;
                            case "Preview":
                                sb_AuditInputXml.Append("<Param Name ='ACTION' Value = 'VIEW PREVIEW' />");
                                break;
                        }
                        sb_AuditInputXml.Append("</AuditParams>");
                        AuditXML = sb_AuditInputXml.ToString();
                    }
                    #endregion

                    ALSolutionCommonObj = new ALSolutionCommon();
                    DownloadPageUrl = ALSolutionCommonObj.EncodeURLForResponseRedirects(sb_DownloadPageUrl.ToString());
                }
            }
            catch (Exception obj_Exception)
            {
                obj_ALErrorCommon.AppendError("DAMFACADE_DAMHELPER_PREPAREDIGITALASSETDOWNLOADURL", this.GetType().ToString(), "PrepareDigitalAssetDownloadUrl", obj_Exception, null);
                return obj_ALErrorCommon.ErrorCollectionXML;
            }
            return null;
        }

        private String GetSessionInfoNode()
        {
            return "<SESSIONINFO TenantName=\"" + TenantName + "\" Language=\"" + CurrentLanguage + "\" UserID=\"" + UserID + "\" SessionID=\"" + SessionID + "\" />";
        }


        /// <alsummary>
        /// <createddate>28 Feb 2008</createddate>
        /// <author>Mr. Pravin Shinde</author>
        /// <purpose>
        /// Get comma separated SectionID, DigitalAssetID, Collection ID List for Search output XML.
        /// </purpose>
        /// <param name="str_SearchoutputXMLPath">Search output XML Path</param>
        /// <param name="xpDoc_SearchOutput">Xpath Document of Search output</param>
        /// <param name="str_SectionIDList">comma seperated SectionID List in () for IN filter</param>
        /// <param name="str_CollectionIDList">comma separated Collection ID List in () for IN filter</param>
        /// <param name="str_DigitlAssetIDList">comma separated DigitalAsset ID List in () for IN filter</param>
        /// </alsummary>
        public ERRORXML GetCommaSeperatedIDList(String str_SearchoutputXMLPath, XPathDocument xpDoc_SearchOutput, ref String str_SectionIDList, ref String str_CollectionIDList, ref String str_DigitlAssetIDList)
        {

            XPathNavigator          xpNav_SearchResults             = null;
            XPathNodeIterator       xpItr_IDList                    = null;
            StringBuilder           sb_DigitalAssetIDList           = null;
            StringBuilder           sb_CollectionIDList             = null;
            StringBuilder           sb_SectionIDList                = null;
            StringBuilder           sb_Xpath                        = null;
            String                  Separator                       = String.Empty;
            ERRORXML                ErrorXML                        = null;

            try
            {

                if (xpDoc_SearchOutput == null && !String.IsNullOrEmpty(str_SearchoutputXMLPath)) 
                {
                    xpDoc_SearchOutput = new XPathDocument(str_SearchoutputXMLPath);
                }

                if (xpDoc_SearchOutput != null)
                {
                    xpNav_SearchResults = xpDoc_SearchOutput.CreateNavigator();
                    sb_Xpath            = new StringBuilder();

                    #region Get DigitalAsset ID List
                    sb_DigitalAssetIDList = new StringBuilder();

                    sb_Xpath.Remove(0, sb_Xpath.Length);
                    sb_Xpath.Append("SEARCHOUTPUTXML/DIGITALASSETCOLLECTION/DIGITALASSET/DIGITALASSETID");
                    xpItr_IDList = xpNav_SearchResults.Select(sb_Xpath.ToString());

                    foreach (XPathNavigator xpNav_ID in xpItr_IDList)
                    {
                        sb_DigitalAssetIDList.AppendFormat("{0}{1}", Separator, xpNav_ID.Value);
                        Separator = ",";
                    }

                    if (sb_DigitalAssetIDList.Length > 0)
                    {
                        sb_DigitalAssetIDList.Insert(0, "(");
                        sb_DigitalAssetIDList.Append(")");
                        str_DigitlAssetIDList = sb_DigitalAssetIDList.ToString();
                    }

                    #endregion

                    #region Get Collection ID List
                    sb_CollectionIDList = new StringBuilder();
                    Separator = String.Empty;

                    sb_Xpath.Remove(0, sb_Xpath.Length);
                    sb_Xpath.Append("SEARCHOUTPUTXML/COLLECTIONCOLLECTION/COLLECTION/COLLECTIONID");
                    xpItr_IDList = xpNav_SearchResults.Select(sb_Xpath.ToString());

                    foreach (XPathNavigator xpNav_ID in xpItr_IDList)
                    {
                        sb_CollectionIDList.AppendFormat("{0}{1}", Separator, xpNav_ID.Value);
                        Separator = ",";
                    }

                    if (sb_CollectionIDList.Length > 0)
                    {
                        sb_CollectionIDList.Insert(0, "(");
                        sb_CollectionIDList.Append(")");
                        str_CollectionIDList = sb_CollectionIDList.ToString();
                    }
                    #endregion

                    #region Get Section ID List
                    sb_SectionIDList = new StringBuilder();
                    Separator = String.Empty;

                    sb_Xpath.Remove(0, sb_Xpath.Length);
                    sb_Xpath.Append("SEARCHOUTPUTXML/SECTIONCOLLECTION/SECTION/SECTIONID");
                    xpItr_IDList = xpNav_SearchResults.Select(sb_Xpath.ToString());

                    foreach (XPathNavigator xpNav_ID in xpItr_IDList)
                    {
                        sb_SectionIDList.AppendFormat("{0}{1}", Separator, xpNav_ID.Value);
                        Separator = ",";
                    }

                    if (sb_SectionIDList.Length > 0)
                    {
                        sb_SectionIDList.Insert(0, "(");
                        sb_SectionIDList.Append(")");
                        str_SectionIDList = sb_SectionIDList.ToString();
                    }
                    #endregion
                }
              
            }
            catch (Exception obj_Exception)
            {
                ErrorXML = obj_Exception.Message;
            }
            return ErrorXML;
        }

       

        public String GetSectionHierarchy(String GetSectionID, bool PrepareTreeXML, bool IsPrepareBreadCrumbXML, String SelectedNode)
        {
            return GetSectionHierarchy(GetSectionID, PrepareTreeXML, IsPrepareBreadCrumbXML, SelectedNode, null);
        }

        public String GetSectionHierarchy(String GetSectionID, bool PrepareTreeXML, bool IsPrepareBreadCrumbXML, String SelectedNode, String GetSectionPath)
        {
            #region Member Varibales
            DAM_SectionFacad            ObjSection          = null;
            SearchConfiguration         ObjSearch           = null;            
            String                      tempTreeXml         = "";
            XmlDocument                 TreeDocument        = null;
            XmlNode                     TreeNode            = null;
            XmlDocument                 SearchDocument      = null;
            XmlDocument                 ConfigDocument      = null;
            XmlNode                     ConfigNode          = null;            
            XmlNode                     tempNode            = null;
            StringBuilder               SearchInput         = null;
            String                      SearchResult        = "";
            String                      SearchError         = "";
            String                      ParentVaultId       = "";
            String                      ParentSectionId     = "";            
            List<String>                ExpandedSectionID   = null;
            //String                      GetSectionPath      = "";
            String                      ParentSectionPath   = "";
            String                      TreeConfigurationXmlFileName = null;
            String                      SubSectionCollection = null;
            XPathDocument               xp_SubSectionCollection = null;
            XPathNavigator              xpnav_SubSectionNav = null;
            #endregion
            //Stopwatch sw = new Stopwatch();
            //Stopwatch temp_sw = new Stopwatch();
            try
            {
                ALCommon.WriteLog(TenantName, "DAM", "DAMmainpage.cs", SessionID, "Begin GetSectionHierarchy");
                //sw.Reset();
                //sw.Start();
                //Initailize objects                
                TreeConfigurationXmlFileName    = ALSolutionCommon.GetConfigurationXMLPath(TenantName, SolutionName, "DAM_AssetHierarchyWidget", "TreeConfiguration");                
                ObjSearch                       = new SearchConfiguration(SessionID, UserID, TenantName,CurrentLanguage);
                ObjSection                      = new DAM_SectionFacad(SessionID, UserID, TenantName, CurrentLanguage);
                ObjSection.SolutionName         = SolutionName;
                ObjSection.WidgetName           = WidgetName;
                SearchInput                     = new StringBuilder(1000);


                #region Get Vault of given section.
                if (String.IsNullOrEmpty(GetSectionPath))
                {
                    //temp_sw.Reset();
                    //temp_sw.Start();
                    SearchInput.Append("<INPUTXML>");
                    SearchInput.AppendFormat("<SESSIONINFO TenantName='{0}' Language='{1}' UserID='{2}' SessionID='{3}'/>", TenantName, CurrentLanguage, UserID, SessionID);
                    SearchInput.Append("<SEARCHCOLLECTION><SEARCH>"); ;
                    SearchInput.Append("<TARGETOBJECT>SECTION</TARGETOBJECT>");
                    SearchInput.Append("<TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>");
                    SearchInput.Append("<SORTCOLUMN>SECTIONID</SORTCOLUMN>");
                    SearchInput.Append("<SORTORDER>ASC</SORTORDER>");
                    SearchInput.Append("<ISCASESENSITIVE>False</ISCASESENSITIVE>");
                    SearchInput.Append("<MAXRESULTSIZE>1</MAXRESULTSIZE>");
                    SearchInput.Append("<SEARCHALLFACETS>true</SEARCHALLFACETS>");
                    SearchInput.Append("<FILTERMODE ApplyFilter=''/>");
                    SearchInput.Append("<FILTERLIST>");
                    SearchInput.Append("<FILTER>");
                    SearchInput.Append("<FIELDNAME>SECTIONID</FIELDNAME>");
                    SearchInput.Append("<OPERATOR>EQUALTO</OPERATOR>");
                    SearchInput.AppendFormat("<FIELDVALUE>{0}</FIELDVALUE>", GetSectionID);
                    SearchInput.AppendFormat("<ISCASESENSITIVE>False</ISCASESENSITIVE>");
                    SearchInput.AppendFormat("<NAVIGATIONPATH>SECTION</NAVIGATIONPATH>");
                    SearchInput.AppendFormat("</FILTER>");
                    SearchInput.AppendFormat("</FILTERLIST>");
                    SearchInput.AppendFormat("<FIELDCOLLECTION>");
                    SearchInput.AppendFormat("<TARGETENTITYFIELDS>");
                    SearchInput.AppendFormat("<FIELD>SECTIONID</FIELD>");
                    SearchInput.AppendFormat("<FIELD>NAME</FIELD>");
                    SearchInput.AppendFormat("<FIELD>SORTORDER</FIELD>");
                    SearchInput.AppendFormat("<FIELD>PATH</FIELD>");
                    SearchInput.AppendFormat("</TARGETENTITYFIELDS>");
                    SearchInput.AppendFormat("<RELATEDENTITYFIELDS NavigationPath='SECTION.SECTION_VAULT'>");
                    SearchInput.AppendFormat("<FIELD>VAULTID</FIELD>");
                    SearchInput.AppendFormat("<FIELD>NAME</FIELD>");
                    SearchInput.AppendFormat("</RELATEDENTITYFIELDS>");
                    SearchInput.AppendFormat("</FIELDCOLLECTION>");
                    SearchInput.AppendFormat("</SEARCH>");
                    SearchInput.AppendFormat("</SEARCHCOLLECTION>");
                    SearchInput.AppendFormat("</INPUTXML>");

                    SearchError = ObjSearch.DoSearch(SearchInput.ToString(), ref SearchResult);
                    if (String.IsNullOrEmpty(SearchError) && !String.IsNullOrEmpty(SearchResult))
                    {
                        SearchDocument      = new XmlDocument();
                        SearchDocument.LoadXml(SearchResult);
                        tempNode            = SearchDocument.SelectSingleNode("descendant::SECTION_VAULT/VAULT/VAULTID");
                        if (tempNode != null)
                            ParentVaultId = tempNode.InnerText;

                        tempNode = SearchDocument.SelectSingleNode("descendant::SECTION/PATH");
                        if (tempNode != null)
                            GetSectionPath = tempNode.InnerText;
                    }
                    else
                        return GetDefaultSectionTree("Error occured while getting section tree.");

                    //temp_sw.Stop();
                    //ALCommon.WriteLog(TenantName, "DAM", "DAMmainpage.cs", SessionID, "Time Required for Get Vault of given section: " + temp_sw.ElapsedMilliseconds.ToString());
                }
                #endregion

                TreeDocument                    = new XmlDocument();

                #region Get RootLevel (vault list) hierarchy and prepare tree.
                //temp_sw.Reset();
                //temp_sw.Start();
                ConfigDocument  = new XmlDocument();
                ConfigDocument.Load(TreeConfigurationXmlFileName);
                ConfigNode      = ConfigDocument.SelectSingleNode("TreeConfiguration/Configuration[@Name='GetSectionTreeDefault']");
                UpdateTreeConfigurationForSectionDisplay(ref ConfigNode, "Section");
                if (ConfigNode != null)
                {
                    String tempResultStr = "";
                    ObjSection.FormattingXSLTPath       = FormattingXSLTPath;
                    ObjSection.ShowSectionAssetCount    = ShowSectionAssetCount;
                    SearchError = ObjSection.GenerateSectionTree(ConfigNode, null, ref tempResultStr);
                    if (String.IsNullOrEmpty(SearchError) && !String.IsNullOrEmpty(tempResultStr))
                    {
                        TreeDocument = new XmlDocument();
                        TreeDocument.LoadXml(tempResultStr);
                    }
                    else
                        return GetDefaultSectionTree("Error occured while getting section tree.");
                }
                //temp_sw.Stop();
                //ALCommon.WriteLog(TenantName, "DAM", "DAMmainpage.cs", SessionID, "Time Required for Get RootLevel (vault list): " + temp_sw.ElapsedMilliseconds.ToString());  
                #endregion

                #region Get subsection list of ParentVaultid and provided section id is subsection of it, prepare tree structure.
                if (TreeDocument != null)
                {
                    ConfigDocument      = new XmlDocument();
                    ConfigDocument.Load(TreeConfigurationXmlFileName);
                    ConfigNode          = ConfigDocument.SelectSingleNode("TreeConfiguration/Configuration[@Name='GetSubSectionTreeByID']");
                    UpdateTreeConfigurationForSectionDisplay(ref ConfigNode, "SubSection");                   
                    ParentSectionPath   = GetSectionPath;
                    ParentSectionId     = ParentVaultId;

                    ExpandedSectionID   = new List<String>();
                    Hashtable ParentSectionIDLevels = new Hashtable();
                    //temp_sw.Reset();
                    //temp_sw.Start();
                    ParentSectionIDLevels = GetIDsOfParentSection(ParentSectionPath,ref SubSectionCollection);
                    //temp_sw.Stop();
                    //ALCommon.WriteLog(TenantName, "DAM", "DAMmainpage.cs", SessionID, "Time Required for GetIDsOfParentSection): " + temp_sw.ElapsedMilliseconds.ToString());  
                    if (ConfigNode != null)
                    {
                        xp_SubSectionCollection = new XPathDocument(new StringReader(SubSectionCollection));
                        xpnav_SubSectionNav     = xp_SubSectionCollection.CreateNavigator();
                        //temp_sw.Reset();
                        //temp_sw.Start();

                        for (int LevelCnt = 1; LevelCnt <= ParentSectionIDLevels.Count; LevelCnt++)
                        {
                            tempTreeXml         = "";
                            ParentSectionId     = ParentSectionIDLevels[LevelCnt].ToString();
                            ALCommon.WriteLog(TenantName, "DAM", "DAMmainpage.cs", SessionID, "Begin GetSubsectionList ");  
                            GetSubsectionList(ConfigNode, ParentSectionId, GetSectionID, ObjSection, xpnav_SubSectionNav,ref tempTreeXml);
                            ALCommon.WriteLog(TenantName, "DAM", "DAMmainpage.cs", SessionID, "End GetSubsectionList ");  
                            
                            TreeNode            = TreeDocument.SelectSingleNode("descendant::TreeNode[@NodeData='" + ParentSectionId + "']");
                            if (TreeNode != null)
                            {
                                ((XmlElement)TreeNode).SetAttribute("ShowExpanded", "True");
                                TreeNode.InnerXml = tempTreeXml;
                            }
                        }
                        //temp_sw.Stop();
                        //ALCommon.WriteLog(TenantName, "DAM", "DAMmainpage.cs", SessionID, "Time Required for - for loop: " + temp_sw.ElapsedMilliseconds.ToString());  

                        TreeNode = TreeDocument.SelectSingleNode("descendant::TreeNode[@NodeData='" + GetSectionID + "']");
                        if (TreeNode != null)
                            ((XmlElement)TreeNode).SetAttribute("ShowExpanded", "True");                           
                    }
                                      
                }
                #endregion
                if (PrepareTreeXML)
                {
                    if (!String.IsNullOrEmpty(TreeDocument.InnerXml))
                        File.WriteAllText(ALSolutionCommon.GetTempPath(SessionID) + "DAM_SectionTreeInput.xml", TreeDocument.InnerXml);
                }
                if (IsPrepareBreadCrumbXML)
                    PrepareBreadCrumbXML(GetSectionID,SelectedNode);

                //sw.Stop();
                //ALCommon.WriteLog(TenantName, "DAM", "DAMmainpage.cs", SessionID, "End GetSectionHierarchy Time Required: " + sw.ElapsedMilliseconds.ToString());  
            }
            catch (Exception )
            {
                return "";
            }
            return TreeDocument.InnerXml;
        }

        public void PrepareBreadCrumbXML(String SelectedSectionID,String SelectedNode)
        {
            XPathDocument           xTreeDoc            = null;
            XPathNavigator          xTreeNavigator      = null;
            XPathNavigator          xCurrntTreeNav      = null;        
            StringBuilder           sb_BreadCrumbXml    = new StringBuilder();
            StringBuilder           sb_Tempxml          = new StringBuilder();
            StringBuilder           BreadCrumbFilePath  = new StringBuilder();
            String                  TreeXMLPath         = null;
            StringBuilder           sb_SelectedNodexml  = new StringBuilder();
            ulong                   UniqueID            = 0;
            String                  str_SectionName     = null;
            String                  str_SectionDisplayName = null;
           
            TreeXMLPath     = ALSolutionCommon.GetTempPath(SessionID) + "DAM_SectionTreeInput.xml";
            if (File.Exists(TreeXMLPath))
            {
                xTreeDoc        = new XPathDocument(TreeXMLPath);
                xTreeNavigator  = xTreeDoc.CreateNavigator();
                xCurrntTreeNav  = xTreeNavigator.SelectSingleNode("descendant::TreeNode[@NodeData='" + SelectedSectionID + "']");

                if (xCurrntTreeNav != null)
                {
                    str_SectionName = xCurrntTreeNav.GetAttribute("SECTIONNAME", "");
                    str_SectionDisplayName = str_SectionName;
                    if (str_SectionName.Length > 15)
                        str_SectionDisplayName = str_SectionName.Substring(0, 15) + "..";

                    if (String.IsNullOrEmpty(SelectedNode))
                        sb_Tempxml.AppendFormat("<BreadCrumb Name=\"{0}\" DisplayText=\"{1}\" Selected=\"True\" RaiseEvent=\"True\" BreadcrumbTextMode=\"Text\" Tooltip=\"{2}\"> </BreadCrumb>", xCurrntTreeNav.GetAttribute("NodeData", ""), ALSolutionCommon.TrimAndReplaceSpecialChars(str_SectionDisplayName), ALSolutionCommon.TrimAndReplaceSpecialChars(str_SectionName));
                    else
                    {
                        ALCommon.GenerateUniqueID(ref UniqueID);
                        sb_SelectedNodexml.AppendFormat("<BreadCrumb Name=\"{0}\" DisplayText=\"{1}\" Selected=\"True\" RaiseEvent=\"True\" BreadcrumbTextMode=\"Text\" Tooltip=\"{2}\"> </BreadCrumb>", UniqueID.ToString(), ALSolutionCommon.TrimAndReplaceSpecialChars(SelectedNode), ALSolutionCommon.TrimAndReplaceSpecialChars(SelectedNode));
                        sb_Tempxml.AppendFormat("<BreadCrumb Name=\"{0}\" DisplayText=\"{1}\" Selected=\"False\" RaiseEvent=\"True\" BreadcrumbTextMode=\"Text\" Tooltip=\"{2}\">{3}</BreadCrumb>", xCurrntTreeNav.GetAttribute("NodeData", ""), ALSolutionCommon.TrimAndReplaceSpecialChars(str_SectionDisplayName), ALSolutionCommon.TrimAndReplaceSpecialChars(str_SectionName), sb_SelectedNodexml.ToString());
                    }

                    while (xCurrntTreeNav.MoveToParent())
                    {
                        sb_BreadCrumbXml.Remove(0, sb_BreadCrumbXml.Length);
                        if (xCurrntTreeNav.GetAttribute("NodeData", "") == "Home")
                        {
                            sb_BreadCrumbXml.AppendFormat("<BreadCrumbs Separator=\">\">{0}</BreadCrumbs>", sb_Tempxml);
                            break;
                        }
                        else
                        {
                            str_SectionName         = xCurrntTreeNav.GetAttribute("SECTIONNAME", "");
                            str_SectionDisplayName  = str_SectionName;
                            if (str_SectionName.Length > 15)
                                str_SectionDisplayName = str_SectionName.Substring(0, 15) + "..";

                            sb_BreadCrumbXml.AppendFormat("<BreadCrumb Name=\"{0}\" DisplayText=\"{1}\" Selected=\"False\" RaiseEvent=\"True\" BreadcrumbTextMode=\"Text\" Tooltip=\"{2}\">{3}</BreadCrumb>", xCurrntTreeNav.GetAttribute("NodeData", ""), ALSolutionCommon.TrimAndReplaceSpecialChars(str_SectionDisplayName), ALSolutionCommon.TrimAndReplaceSpecialChars(str_SectionName), sb_Tempxml);
                        }

                        sb_Tempxml.Remove(0, sb_Tempxml.Length);
                        sb_Tempxml.Append(sb_BreadCrumbXml);
                    }
                    if(String.IsNullOrEmpty(BreadCrumbControlInputFilePath))
                        BreadCrumbFilePath.AppendFormat("{0}Assetlist_BreadCrumbControlInput.xml", ALSolutionCommon.GetTempPath(SessionID));
                    else
                        BreadCrumbFilePath.Append(BreadCrumbControlInputFilePath);

                    File.WriteAllText(BreadCrumbFilePath.ToString(), sb_BreadCrumbXml.ToString());
                }
            }
        }

        /// <summary>
        ///  This method is used to Update configuration node with specified section tree display configuration. 
        /// This method works with DOSearch configuration
        /// </summary>
        /// <param name="ConfigurationNode">It hold tree display configuration node</param>        
        /// <param name="Type">Section or Subsection</param>
        
        public void UpdateTreeConfigurationForSectionDisplay(ref XmlNode ConfigurationNode, String Type)
        {
            #region Member variables
            XmlNode         TreeNode                = null;
            XmlNode         TextDataColumnNode      = null;
            XmlNode         TooltipDataColumnNode   = null;
            XmlNode         SearchFieldsNode        = null;
            XmlNode         TempFieldsNode          = null;                        
            XmlNode         RelatedEntityFacetNode  = null;       
            XmlNode         AdditionalInfoNode      = null;

            XPathDocument   xp_SectionTreeDisplayConfig     = null;
            XPathNavigator  xpNav_SectionTreeDisplayConfig  = null;
            XPathNavigator  xpNav_TextDataColumn            = null;
            XPathNavigator  xpNav_ToolTipDataColumn         = null;
            XPathNavigator  xpNav_SearchFields              = null;
            XPathNavigator  xpNav_Fields                    = null;
            XPathNodeIterator xpItr_RelatedEntityFacets     = null;
            XPathNavigator  xpNav_RelatedEntityFacets       = null;
            XPathNavigator  xpNav_AdditionalInfo            = null;
            #endregion

            //ALCommon.WriteLog(SetBaseObj.TenantName, objALParam.SolutionName, "Section", objALParam.SessionID, "Start Config update");
            if (!String.IsNullOrEmpty(SectionTreeDisplayConfiguration) && File.Exists(SectionTreeDisplayConfiguration))
            {
                xp_SectionTreeDisplayConfig = new XPathDocument(SectionTreeDisplayConfiguration);
                xpNav_SectionTreeDisplayConfig = xp_SectionTreeDisplayConfig.CreateNavigator();
            }         
            
            //Update configuration node with section tree display configuration specified. 
            if (xpNav_SectionTreeDisplayConfig != null)
            {
                TreeNode                = ConfigurationNode.SelectSingleNode("DisplayLayout/Tree/DisplayNodes/TreeNode");
                SearchFieldsNode        = ConfigurationNode.SelectSingleNode("SearchFields");

                if(TreeNode != null)
                {
                    #region Add TextDatacolumn in tree configuration, if specified
                    xpNav_TextDataColumn                = xpNav_SectionTreeDisplayConfig.SelectSingleNode("SectionTreeDisplayConfiguration/TextDataColumn");
                    if(xpNav_TextDataColumn != null && !String.IsNullOrEmpty(xpNav_TextDataColumn.Value))
                    {
                        ((XmlElement)TreeNode).SetAttribute("TextDataColumn", "");
                        TextDataColumnNode              = (XmlNode)ConfigurationNode.OwnerDocument.CreateElement("TextDataColumn");
                        TextDataColumnNode.InnerXml     = xpNav_TextDataColumn.InnerXml;
                        TreeNode.AppendChild(TextDataColumnNode);
                    }
                    #endregion

                    #region Add TolltipDatacolumn in tree configuration, if specified
                    xpNav_ToolTipDataColumn             = xpNav_SectionTreeDisplayConfig.SelectSingleNode("SectionTreeDisplayConfiguration/ToolTipDataColumn");
                    if (xpNav_ToolTipDataColumn != null && !String.IsNullOrEmpty(xpNav_ToolTipDataColumn.Value))
                    {
                        ((XmlElement)TreeNode).SetAttribute("ToolTipDataColumn", "");
                        TooltipDataColumnNode           = (XmlNode)ConfigurationNode.OwnerDocument.CreateElement("ToolTipDataColumn");
                        TooltipDataColumnNode.InnerXml  = xpNav_ToolTipDataColumn.InnerXml;
                        TreeNode.AppendChild(TooltipDataColumnNode);
                    }
                    #endregion
                }

                #region Add SearchFields(required for TextDataColumn and TooltipDataColumn) in configuration.
                xpNav_SearchFields = xpNav_SectionTreeDisplayConfig.SelectSingleNode("SectionTreeDisplayConfiguration/SearchFields[@Type='" + Type + "']");
                if (xpNav_SearchFields != null)
                { 
                    XmlDocument ConfigurationDoc = ConfigurationNode.OwnerDocument;
                    //Check if Target entity fields
                    xpNav_Fields                    = xpNav_SearchFields.SelectSingleNode("TargetObject");
                    if (xpNav_Fields != null)
                    {
                        TempFieldsNode              = SearchFieldsNode.SelectSingleNode("TargetObject");
                        if (TempFieldsNode == null)
                        {
                            TempFieldsNode  = (XmlNode)ConfigurationDoc.CreateElement("TargetObject");
                            ((XmlElement)TempFieldsNode).SetAttribute("ObjectName", xpNav_Fields.GetAttribute("ObjectName", ""));
                            ((XmlElement)TempFieldsNode).SetAttribute("NavigationPath", xpNav_Fields.GetAttribute("NavigationPath", ""));
                            SearchFieldsNode.AppendChild(TempFieldsNode);
                        }
                        AddSearchFieldNode(xpNav_Fields, ref TempFieldsNode, ConfigurationDoc);
                    }

                    //Check if Related entity fields                                        
                    xpNav_RelatedEntityFacets       = xpNav_SearchFields.SelectSingleNode("RelatedEntities");
                    if (xpNav_RelatedEntityFacets != null)
                    {
                        xpItr_RelatedEntityFacets   = xpNav_RelatedEntityFacets.Select("RelatedEntity");
                        if (xpItr_RelatedEntityFacets.Count > 0)
                        {
                            RelatedEntityFacetNode  = SearchFieldsNode.SelectSingleNode("RelatedEntities");
                            if (RelatedEntityFacetNode == null)
                            {
                                RelatedEntityFacetNode = (XmlNode)ConfigurationDoc.CreateElement("RelatedEntities");
                                SearchFieldsNode.AppendChild(RelatedEntityFacetNode);
                            }

                            foreach (XPathNavigator xpNav_RelatedEntityNode in xpItr_RelatedEntityFacets)
                            {
                                if (xpNav_RelatedEntityNode.Select("Column").Count > 0)
                                {
                                    TempFieldsNode = RelatedEntityFacetNode.SelectSingleNode("RelatedEntity[@NavigationPath=" + ALXmlUtil.GetXpathValueString(xpNav_RelatedEntityNode.GetAttribute("NavigationPath", "")) + "]");
                                    if (TempFieldsNode == null)
                                    {
                                        TempFieldsNode = (XmlNode)ConfigurationDoc.CreateElement("RelatedEntity");
                                        ((XmlElement)TempFieldsNode).SetAttribute("ObjectName", xpNav_RelatedEntityNode.GetAttribute("ObjectName", ""));
                                        ((XmlElement)TempFieldsNode).SetAttribute("NavigationPath", xpNav_RelatedEntityNode.GetAttribute("NavigationPath", ""));
                                        RelatedEntityFacetNode.AppendChild(TempFieldsNode);
                                    }
                                    AddSearchFieldNode(xpNav_RelatedEntityNode, ref TempFieldsNode, ConfigurationDoc);
                                }
                            }
                        }
                    }

                    //Check if Related facet fields
                    xpNav_RelatedEntityFacets       = xpNav_SearchFields.SelectSingleNode("RelatedFacets");
                    if (xpNav_RelatedEntityFacets != null)
                    {
                        xpItr_RelatedEntityFacets   = xpNav_RelatedEntityFacets.Select("RelatedFacet");
                        if (xpItr_RelatedEntityFacets.Count > 0)
                        {
                            RelatedEntityFacetNode  = SearchFieldsNode.SelectSingleNode("RelatedFacets");
                            if (RelatedEntityFacetNode == null)
                            {
                                RelatedEntityFacetNode = (XmlNode)ConfigurationDoc.CreateElement("RelatedFacets");
                                SearchFieldsNode.AppendChild(RelatedEntityFacetNode);
                            }

                            foreach (XPathNavigator xpNav_RelatedFacetNode in xpItr_RelatedEntityFacets)
                            {
                                if (xpNav_RelatedFacetNode.Select("Column").Count > 0)
                                {
                                    TempFieldsNode = RelatedEntityFacetNode.SelectSingleNode("RelatedFacet[@NavigationPath=" + ALXmlUtil.GetXpathValueString(xpNav_RelatedFacetNode.GetAttribute("NavigationPath", "")) + "]");
                                    if (TempFieldsNode == null)
                                    {
                                        TempFieldsNode = (XmlNode)ConfigurationDoc.CreateElement("RelatedFacet");
                                        ((XmlElement)TempFieldsNode).SetAttribute("ObjectName", xpNav_RelatedFacetNode.GetAttribute("ObjectName", ""));
                                        ((XmlElement)TempFieldsNode).SetAttribute("NavigationPath", xpNav_RelatedFacetNode.GetAttribute("NavigationPath", ""));
                                        RelatedEntityFacetNode.AppendChild(TempFieldsNode);
                                    }
                                    AddSearchFieldNode(xpNav_RelatedFacetNode, ref TempFieldsNode, ConfigurationDoc);
                                }
                            }
                        }
                    }
                }
                #endregion
                    
                #region Add Additional Field information columns to tree configuration
                AdditionalInfoNode      = ConfigurationNode.SelectSingleNode("DisplayLayout/Tree/AdditionalFieldInformation");
                xpNav_AdditionalInfo    = xpNav_SectionTreeDisplayConfig.SelectSingleNode("SectionTreeDisplayConfiguration/AdditionalFieldInformation");
                if (xpNav_AdditionalInfo != null && AdditionalInfoNode != null)
                    AdditionalInfoNode.InnerXml = xpNav_AdditionalInfo.InnerXml;
                #endregion                
            }
            //ALCommon.WriteLog(SetBaseObj.TenantName, objALParam.SolutionName, "Section", objALParam.SessionID, "End Config update");
        }

        /// <alsummary>
        /// <createddate>22 July 2008</createddate>
        /// <author>Mr. Pravin Shinde</author>
        /// <purpose>
        /// Checks access for parent section hierarchy. 
        /// if 
        /// 1. DigitalAsset/Collection/Section is not having access on any section in parent section hierarchy 
        /// IsAllowed = false;
        /// else
        /// IsAllowed = true;
        /// </purpose>
        /// </summary>
        /// <param name="contentType">Section/Collection/DigitalAsset</param>
        /// <param name="ObjectID"></param>
        /// <param name="ParentSectionPath"></param>
        /// <param name="IsAllowed"></param>
        /// <returns></returns>
        public ERRORXML CheckAccessOnParentSectionHierarchy(SectionContentType contentType, String ObjectID, String ParentSectionPath, out Boolean IsAllowed)
        {

            XPathDocument       xpDoc_SearchResult      = null;
            XPathNavigator      xpNav_SearchResult      = null;
            XPathNavigator      xpNav_SectionPath       = null;
            XPathNodeIterator   xpItr_SectionCollection = null;
            StringBuilder       sb_SearchHeader         = null;
            StringBuilder       sb_Filter               = null;
            StringBuilder       sb_SearchInputXML       = null;
            String              str_ObjectTypePK        = null;
            String              str_RelationshipName    = null;
            String              str_SearchOutputXML     = null;
            String              str_FilterString        = null;
            ERRORXML            ErrorXML                = null; 
            String[]            strarr_SectionPath      = null;
            String[]            Separator               = {"\\"};
            StringBuilder       sb_SectionPath          = null;
            ALErrorCommon       obj_ALErrorCommon       = null;

            try
            {
                obj_Search          = new SearchConfiguration(SessionID, UserID, TenantName, CurrentLanguage);
                IsAllowed           = false;
                obj_ALErrorCommon   = new ALErrorCommon();
                sb_SearchInputXML   = new StringBuilder(900);
                sb_SectionPath      = new StringBuilder(200);
                sb_Filter           = new StringBuilder(300);

                #region Prepare Search Input XML
                sb_SearchHeader = new StringBuilder(500);
                sb_SearchHeader.AppendFormat(@"<SEARCH>
	                                            <TARGETOBJECT>SECTION</TARGETOBJECT>
		                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
		                                            <SESSIONINFO TenantName=""{0}"" Language=""{1}"" UserID=""{2}"" SessionID=""{3}""/>
		                                            <AUTHORIZATIONREQUIRED>TRUE</AUTHORIZATIONREQUIRED>
		                                            <TARGETENTITY>
			                                            <PRIVILEGENAME>READ</PRIVILEGENAME>
			                                            <SELECTLIST>
				                                            <EXPRESSION>
					                                            <FIELDNAME>SECTIONID</FIELDNAME>
					                                            <ALIAS>SECTIONID</ALIAS>
				                                            </EXPRESSION>
				                                            <EXPRESSION>
					                                            <FIELDNAME>PATH</FIELDNAME>
					                                            <ALIAS>PATH</ALIAS>
				                                            </EXPRESSION>
			                                            </SELECTLIST>", TenantName, CurrentLanguage, UserID, SessionID);


                #endregion

                #region Get Parent Section Path
                if (String.IsNullOrEmpty(ParentSectionPath))
                {
                    switch (contentType)
                    {
                        case SectionContentType.DigitalAsset:
                            {
                                str_ObjectTypePK        = "DIGITALASSETID";
                                str_RelationshipName    = "SECTION_DIGITALASSET";
                            } break;
                        case SectionContentType.Collection:
                            {
                                str_ObjectTypePK        = "COLLECTIONID";
                                str_RelationshipName    = "SECTION_COLLECTION";
                            } break;
                        case SectionContentType.Section:
                            {
                                str_ObjectTypePK        = "SECTIONID";
                                str_RelationshipName    = "SECTION_SUBSECTION";
                            } break;
                    }

                 
                    sb_Filter.AppendFormat(@"<FILTERLIST>
			                                   <FILTER>
				                                 <EXPRESSION>{0}</EXPRESSION>
				                                 <OPERATOR>EQUALTO</OPERATOR>
				                                 <EXPRESSIONVALUE>{1}</EXPRESSIONVALUE>
				                                 <ISCASESENSITIVE>False</ISCASESENSITIVE>
				                                 <RELATIONSHIPNAME>{2}</RELATIONSHIPNAME>
			                                    </FILTER>
		                                     </FILTERLIST>", str_ObjectTypePK, ObjectID, str_RelationshipName);
                    sb_SearchInputXML.AppendFormat("{0}{1}{2}", sb_SearchHeader, sb_Filter, "</TARGETENTITY></SEARCH>");

                    ErrorXML = obj_Search.GetHierarchialData(sb_SearchInputXML.ToString(), ref str_SearchOutputXML);
                    if (!String.IsNullOrEmpty(ErrorXML))
                    {
                        obj_ALErrorCommon.AppendError("DAMFACADE_DAMHELPER_PARENTSECTIONPATHREADFAILED", this.GetType().ToString(), "CheckAccessOnParentSectionHierarchy", null, ErrorXML);
                        return obj_ALErrorCommon.ErrorCollectionXML;
                    }
                    xpDoc_SearchResult  = new XPathDocument(new StringReader(str_SearchOutputXML));
                    xpNav_SearchResult  = xpDoc_SearchResult.CreateNavigator();
                    xpNav_SectionPath   = xpNav_SearchResult.SelectSingleNode("SECTIONCOLLECTION/SECTION/PATH");
                    //User is not having access on its parent section 
                    if (xpNav_SectionPath != null)
                    {
                        ParentSectionPath = xpNav_SectionPath.Value;
                    }
                    else
                    {
                        return null;
                    }
                    
                }

                #endregion

                #region Check access

                #region Get Parent Sections
                str_FilterString =@"<FILTER>
				                     <EXPRESSION>PATH</EXPRESSION>
				                     <OPERATOR>EQUALTO</OPERATOR>
				                     <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
				                     <ISCASESENSITIVE>False</ISCASESENSITIVE>
			                      </FILTER>";

                sb_Filter.Remove(0, sb_Filter.Length);
                sb_Filter.Append("<FILTERLIST>");

                strarr_SectionPath = ParentSectionPath.Split(Separator, StringSplitOptions.RemoveEmptyEntries);
                Separator[0] = "";
                for (int count = 0; count < strarr_SectionPath.Length; count++)
                {
                    sb_SectionPath.AppendFormat("{0}{1}",Separator[0], ALSolutionCommon.TrimAndReplaceSpecialChars(strarr_SectionPath[count])); 
                    sb_Filter.AppendFormat(str_FilterString, sb_SectionPath.ToString());
                    Separator[0] = "\\";
                }

                sb_Filter.Append("</FILTERLIST>");
                str_SearchOutputXML = String.Empty;
                sb_SearchInputXML.Remove(0, sb_SearchInputXML.Length);
                sb_SearchInputXML.AppendFormat("{0}{1}</TARGETENTITY></SEARCH>", sb_SearchHeader,sb_Filter);

                ErrorXML = obj_Search.GetHierarchialData(sb_SearchInputXML.ToString(),ref str_SearchOutputXML);
                if (!String.IsNullOrEmpty(ErrorXML))
                {
                    obj_ALErrorCommon.AppendError("DAMFACADE_DAMHELPER_PARENTSECTIONHIERARCHYREADFAILED", this.GetType().ToString(), "CheckAccessOnParentSectionHierarchy", null, ErrorXML);
                    return obj_ALErrorCommon.ErrorCollectionXML;
                }
                xpDoc_SearchResult      = new XPathDocument(new StringReader(str_SearchOutputXML));
                xpNav_SearchResult      = xpDoc_SearchResult.CreateNavigator();
                xpItr_SectionCollection = xpNav_SearchResult.Select("SECTIONCOLLECTION/SECTION");
                #endregion

                if (xpItr_SectionCollection.Count == strarr_SectionPath.Length)
                {
                    IsAllowed = true;
                }
                #endregion
            }
            catch (Exception obj_Exception)
            {
                IsAllowed = false;
                obj_ALErrorCommon.AppendError("DAMFACADE_DAMHELPER_CHECKACCESSONPARENTSECTIONHIERARCHYFAILED", this.GetType().ToString(), "CheckAccessOnParentSectionHierarchy", obj_Exception, null);
                return obj_ALErrorCommon.ErrorCollectionXML;
            }

            return null; 
        }

        private void AddSearchFieldNode(XPathNavigator xpNav_ConfigFieldsNode, ref XmlNode SearchFieldNode, XmlDocument ConfigDoc)
        {
            XmlNode ColumnNode = null;

            foreach (XPathNavigator xpNav_Columns in xpNav_ConfigFieldsNode.Select("Column"))
            {
                ColumnNode = SearchFieldNode.SelectSingleNode("Column[@Name=" + ALXmlUtil.GetXpathValueString(xpNav_Columns.GetAttribute("Name", "")) + "]");
                if (ColumnNode == null)
                {
                    ColumnNode = (XmlNode)ConfigDoc.CreateElement("Column");
                    ((XmlElement)ColumnNode).SetAttribute("Name", xpNav_Columns.GetAttribute("Name", ""));
                    SearchFieldNode.AppendChild(ColumnNode);
                }
            }
        }

        /// <summary>
        ///  This method is used to Update configuration node with specified section tree display configuration. 
        /// This method works with NSearch configuration
        /// </summary>
        /// <param name="ConfigurationNode">It hold tree display configuration node</param>
        /// <param name="SearchInputXML">NSearch input XML</param>
        /// <param name="Type">Section or Subsection</param>

        public String UpdateTreeConfigurationForSectionDisplay_NSearch(ref XmlNode ConfigurationNode, String SearchInputXML, String Type)
        { 
            #region Member variables                      
            XPathDocument   xp_SectionTreeDisplayConfig     = null;
            XPathNavigator  xpNav_SectionTreeDisplayConfig  = null;            
            XPathNavigator  xpNav_SearchFields              = null;
            XPathNavigator  xpNav_Fields                    = null;
            XPathNodeIterator xpItr_RelatedEntityFacets     = null;
            XPathNavigator  xpNav_RelatedEntityFacets       = null;            

            XmlDocument     SearchInputDoc                  = null;                        
            XmlNode         SectionNode                     = null;
            XmlNode         SelectlistNode                  = null;
            XmlNode         RelationNode                    = null;
            XmlNode         TreeNode                        = null;
            XmlNode         TextDataColumnNode              = null;
            XmlNode         TooltipDataColumnNode           = null;
            XmlNode         SearchFieldsNode                = null;
            XmlNode         AdditionalInfoNode              = null;
            XPathNavigator  xpNav_TextDataColumn            = null;
            XPathNavigator  xpNav_ToolTipDataColumn         = null;
            XPathNavigator  xpNav_AdditionalInfo            = null;
            #endregion

            SearchInputDoc = new XmlDocument();
            SearchInputDoc.LoadXml(SearchInputXML);

            //ALCommon.WriteLog(SetBaseObj.TenantName, objALParam.SolutionName, "Section", objALParam.SessionID, "Start Config update");
            if (!String.IsNullOrEmpty(SectionTreeDisplayConfiguration) && File.Exists(SectionTreeDisplayConfiguration))
            {
                xp_SectionTreeDisplayConfig     = new XPathDocument(SectionTreeDisplayConfiguration);
                xpNav_SectionTreeDisplayConfig  = xp_SectionTreeDisplayConfig.CreateNavigator();
            }

            //Add SearchFields(required for TextDataColumn and TooltipDataColumn) in configuration.
            if (xpNav_SectionTreeDisplayConfig != null)
            {
                if (Type == "Section")
                {
                    if (SearchInputDoc.DocumentElement.LocalName == "SEARCHCOLLECTION")
                        SectionNode = SearchInputDoc.SelectSingleNode("SEARCHCOLLECTION/SEARCH[TARGETOBJECT/text()='SECTION']/TARGETENTITY");
                    else
                        SectionNode = SearchInputDoc.SelectSingleNode("SEARCH[TARGETOBJECT/text()='SECTION']/TARGETENTITY");
                }
                else if (Type == "SubSection")
                {
                    if (SearchInputDoc.DocumentElement.LocalName == "SEARCHCOLLECTION")
                        SectionNode = SearchInputDoc.SelectSingleNode("SEARCHCOLLECTION/SEARCH/TARGETENTITY/RELATIONSHIP[@NAME='SECTION_SUBSECTION']");
                    else
                        SectionNode = SearchInputDoc.SelectSingleNode("SEARCH/TARGETENTITY/RELATIONSHIP[@NAME='SECTION_SUBSECTION']");
                }

                if (SectionNode != null)
                { 
                    
                     xpNav_SearchFields    = xpNav_SectionTreeDisplayConfig.SelectSingleNode("SectionTreeDisplayConfiguration/SearchFields[@Type='" + Type + "']");
                     if (xpNav_SearchFields != null)
                     {
                         #region Add Target Entity fields
                         xpNav_Fields = xpNav_SearchFields.SelectSingleNode("TargetObject");
                         if (xpNav_Fields != null)
                         {
                             SelectlistNode = SectionNode.SelectSingleNode("SELECTLIST");
                             AddSearchFieldNodeToXml(xpNav_Fields, ref SelectlistNode, SearchInputDoc);
                         }
                         #endregion
                         #region Check if Related entity fields                                        
                         xpNav_RelatedEntityFacets       = xpNav_SearchFields.SelectSingleNode("RelatedEntities");
                         if (xpNav_RelatedEntityFacets != null)
                         {
                            xpItr_RelatedEntityFacets   = xpNav_RelatedEntityFacets.Select("RelatedEntity");
                            if (xpItr_RelatedEntityFacets.Count > 0)
                            {
                                foreach (XPathNavigator xpNav_RelatedEntityNode in xpItr_RelatedEntityFacets)
                                {
                                    if (xpNav_RelatedEntityNode.Select("Column").Count > 0)
                                    {
                                        RelationNode = SectionNode.ParentNode.SelectSingleNode("RELATIONSHIP[@NAME=" + ALXmlUtil.GetXpathValueString(xpNav_RelatedEntityNode.GetAttribute("RelationName", "")) + "]");
                                        if (RelationNode == null)
                                        {
                                            RelationNode = (XmlNode)SearchInputDoc.CreateElement("RELATIONSHIP");
                                            ((XmlElement)RelationNode).SetAttribute("NAME", xpNav_RelatedEntityNode.GetAttribute("RelationName", ""));
                                            SectionNode.AppendChild(RelationNode);
                                        }

                                        SelectlistNode = RelationNode.SelectSingleNode("SELECTLIST");
                                        if (SelectlistNode == null)
                                        {
                                            SelectlistNode = (XmlNode)SearchInputDoc.CreateElement("SELECTLIST");
                                            RelationNode.AppendChild(SelectlistNode);
                                        }
                                        AddSearchFieldNodeToXml(xpNav_RelatedEntityNode, ref SelectlistNode, SearchInputDoc);
                                    }
                                }
                            }
                         }
                        #endregion
                         #region Check if Related Facets fields                                        
                         xpNav_RelatedEntityFacets       = xpNav_SearchFields.SelectSingleNode("RelatedFacets");
                         if (xpNav_RelatedEntityFacets != null)
                         {
                            xpItr_RelatedEntityFacets   = xpNav_RelatedEntityFacets.Select("RelatedFacet");
                            if (xpItr_RelatedEntityFacets.Count > 0)
                            {
                                foreach (XPathNavigator xpNav_RelatedEntityNode in xpItr_RelatedEntityFacets)
                                {
                                    if (xpNav_RelatedEntityNode.Select("Column").Count > 0)
                                    {
                                        RelationNode = SectionNode.SelectSingleNode("RELATIONSHIP[@NAME=" + ALXmlUtil.GetXpathValueString(xpNav_RelatedEntityNode.GetAttribute("RelationName", "")) + "]");
                                        if (RelationNode == null)
                                        {
                                            RelationNode = (XmlNode)SearchInputDoc.CreateElement("RELATIONSHIP");
                                            ((XmlElement)RelationNode).SetAttribute("NAME", xpNav_RelatedEntityNode.GetAttribute("RelationName", ""));
                                            SectionNode.AppendChild(RelationNode);
                                        }

                                        SelectlistNode = RelationNode.SelectSingleNode("SELECTLIST");
                                        if (SelectlistNode == null)
                                        {
                                            SelectlistNode = (XmlNode)SearchInputDoc.CreateElement("SELECTLIST");
                                            RelationNode.AppendChild(SelectlistNode);
                                        }
                                        AddSearchFieldNodeToXml(xpNav_RelatedEntityNode, ref SelectlistNode, SearchInputDoc);
                                    }
                                }
                            }
                         }
                        #endregion
                     }
                }

                //Update configuration node with section tree display configuration specified. 
                if (ConfigurationNode != null)
                {
                    TreeNode = ConfigurationNode.SelectSingleNode("DisplayLayout/Tree/DisplayNodes/TreeNode");
                    SearchFieldsNode = ConfigurationNode.SelectSingleNode("SearchFields");

                    if (TreeNode != null)
                    {
                        #region Add TextDatacolumn in tree configuration, if specified
                        xpNav_TextDataColumn = xpNav_SectionTreeDisplayConfig.SelectSingleNode("SectionTreeDisplayConfiguration/TextDataColumn");
                        if (xpNav_TextDataColumn != null && !String.IsNullOrEmpty(xpNav_TextDataColumn.Value))
                        {
                            ((XmlElement)TreeNode).SetAttribute("TextDataColumn", "");
                            TextDataColumnNode = (XmlNode)ConfigurationNode.OwnerDocument.CreateElement("TextDataColumn");
                            TextDataColumnNode.InnerText = xpNav_TextDataColumn.Value;
                            TreeNode.AppendChild(TextDataColumnNode);
                        }
                        #endregion

                        #region Add TolltipDatacolumn in tree configuration, if specified
                        xpNav_ToolTipDataColumn = xpNav_SectionTreeDisplayConfig.SelectSingleNode("SectionTreeDisplayConfiguration/ToolTipDataColumn");
                        if (xpNav_ToolTipDataColumn != null && !String.IsNullOrEmpty(xpNav_ToolTipDataColumn.Value))
                        {
                            ((XmlElement)TreeNode).SetAttribute("ToolTipDataColumn", "");
                            TooltipDataColumnNode = (XmlNode)ConfigurationNode.OwnerDocument.CreateElement("ToolTipDataColumn");
                            TooltipDataColumnNode.InnerText = xpNav_ToolTipDataColumn.Value;
                            TreeNode.AppendChild(TooltipDataColumnNode);
                        }
                        #endregion

                        #region Add Additional Field information columns to tree configuration
                        AdditionalInfoNode = ConfigurationNode.SelectSingleNode("DisplayLayout/Tree/AdditionalFieldInformation");
                        xpNav_AdditionalInfo = xpNav_SectionTreeDisplayConfig.SelectSingleNode("SectionTreeDisplayConfiguration/AdditionalFieldInformation");
                        if (xpNav_AdditionalInfo != null && AdditionalInfoNode != null)
                            AdditionalInfoNode.InnerXml = xpNav_AdditionalInfo.InnerXml;
                        #endregion
                    }
                }
            }
            return SearchInputDoc.InnerXml;
        }

        private void AddSearchFieldNodeToXml(XPathNavigator xpNav_ConfigFieldsNode, ref XmlNode SearchFieldNode, XmlDocument SearchInputDoc)
        {
            XmlNode     ColumnNode      = null;
            XmlNode     TempFieldNode   = null;

            foreach (XPathNavigator xpNav_Columns in xpNav_ConfigFieldsNode.Select("Column"))
             {
                 ColumnNode = SearchFieldNode.SelectSingleNode("EXPRESSION/FIELDNAME[text()=" + ALXmlUtil.GetXpathValueString(xpNav_Columns.GetAttribute("Name", "")) + "]");
                 if (ColumnNode == null)
                 {
                     ColumnNode                 = (XmlNode)SearchInputDoc.CreateElement("EXPRESSION");                                     
                     TempFieldNode              = (XmlNode)SearchInputDoc.CreateElement("FIELDNAME");
                     TempFieldNode.InnerText    = xpNav_Columns.GetAttribute("Name", "");
                     ColumnNode.AppendChild(TempFieldNode);
                     TempFieldNode              = (XmlNode)SearchInputDoc.CreateElement("ALIAS");
                     TempFieldNode.InnerText    = xpNav_Columns.GetAttribute("Name", "");
                     ColumnNode.AppendChild(TempFieldNode);
                     SearchFieldNode.AppendChild(ColumnNode);
                 }
             }        
        }

        public String GetDamJobErrorDetails(String str_JobID, ref String str_SearchOpXml)
        {
            #region LocalVariables
            String                  str_SearchConfigurationPath     = null;
            String                  str_SearchInputXml              = null;
            XPathDocument           xpDoc_SearchConfiguration       = null;
            XPathNavigator          xpNav_SearchConfiguration       = null;
            XPathNavigator          xpNav_ConfigurationNode         = null;
            #endregion

            str_SearchConfigurationPath = ALSolutionCommon.GetConfigurationXMLPath(TenantName, SolutionName, "", "SearchConfiguration");
            xpDoc_SearchConfiguration   = new XPathDocument(str_SearchConfigurationPath);
            xpNav_SearchConfiguration   = xpDoc_SearchConfiguration.CreateNavigator();
            xpNav_ConfigurationNode     = xpNav_SearchConfiguration.SelectSingleNode("PredefinedSearchConfigurations/Configuration[@Name='JobError']");
            if (xpNav_ConfigurationNode != null)
            {
                str_SearchInputXml  = xpNav_ConfigurationNode.SelectSingleNode("SEARCH").OuterXml.Replace("$AL$LANGUAGE$AL$", CurrentLanguage);
                str_SearchInputXml  = str_SearchInputXml.Replace("$AL$TENANTNAME$AL$", TenantName);
                str_SearchInputXml  = str_SearchInputXml.Replace("$AL$USERID$AL$", UserID);
                str_SearchInputXml  = str_SearchInputXml.Replace("$AL$SessionID$AL$", SessionID);
                str_SearchInputXml  = str_SearchInputXml.Replace("$AL$DAMJOBID$AL$", str_JobID);
                obj_Search          = new SearchConfiguration(SessionID, UserID, TenantName, CurrentLanguage);
                ErrorXML            = obj_Search.GetHierarchialData(str_SearchInputXml, ref str_SearchOpXml);
            }
            return ErrorXML;
        }

        public ERRORXML CheckForCrossVaultFunctionality(String AssetIDList, Boolean IsTermsAndConditionsEnabled, Boolean IsReasonForDownloadEnabled, out Boolean IsCrossVaultAllowed, out Boolean IsTandCSRequired, out Boolean isUsedDefaultTandCS,out Boolean isVaultSpecificRFDUsed, out String VaultCollection,ref String VaultIDForReasonForDownload)
        {
            StringBuilder       sb_SearchInputXML           = null;
            XPathDocument       xpDoc_DigitalAssets         = null;
            XPathNavigator      xpNav_RootDigitalAssets     = null;
            XPathNodeIterator   xpNodItr_DigitalAssets      = null;
            XPathNodeIterator   xpNodItr_Collections        = null;
            XPathDocument       xpDoc_VaultList             = null;
            XPathNavigator      xpNav_RootVaultList         = null;
            XPathNavigator      xpNav_Vault_TermsAndCSType  = null;
            XPathNodeIterator   xpNodItr_VaultList          = null; 
            StringBuilder       sb_DAIDList                 = null;
            StringBuilder       sb_CollectionIDList         = null;
            String              Separator                   = String.Empty;
            SearchConfiguration obj_SearchConfiguration     = null;
            ERRORXML            ErrorXML                    = null;
            String              SearchOutPutXML             = null;
            Boolean             IsVaultUsingDefaultTandCS   = false;
            Boolean             IsVaultNotRequiredTandCS    = false;
            String              VaultID                     = null;
            StringBuilder       sb_FormName                 = null;
            String              str_FormConfigXMLPath       = null;
            IsCrossVaultAllowed     = false;
            IsTandCSRequired        = true;
            isUsedDefaultTandCS     = false;
            isVaultSpecificRFDUsed  = false;
            VaultCollection         = String.Empty;
            List<String> VaultList = new List<String>();
            StringBuilder sb_VaultListXML = null;
            try
            {
                xpDoc_DigitalAssets     = new XPathDocument(new StringReader(AssetIDList));
                xpNav_RootDigitalAssets = xpDoc_DigitalAssets.CreateNavigator();
                sb_DAIDList             = new StringBuilder(100);
                xpNodItr_DigitalAssets  = xpNav_RootDigitalAssets.Select("SEARCHCOLLECTION/DIGITALASSETCOLLECTION/DIGITALASSET/DIGITALASSETID");
                foreach (XPathNavigator xpNav_DigitalAssetID in xpNodItr_DigitalAssets)
                {
                    sb_DAIDList.AppendFormat("{0}{1}", Separator, xpNav_DigitalAssetID.Value);
                    Separator = ",";
                }

                Separator               = String.Empty;
                sb_CollectionIDList     = new StringBuilder(100);
                xpNodItr_Collections = xpNav_RootDigitalAssets.Select("SEARCHCOLLECTION/COLLECTIONLIST/COLLECTION/COLLECTIONID");
                foreach (XPathNavigator xpNav_Collection in xpNodItr_Collections)
                {
                    sb_CollectionIDList.AppendFormat("{0}{1}", Separator, xpNav_Collection.Value);
                    Separator = ",";
                }

                sb_SearchInputXML = new StringBuilder(500);
                sb_SearchInputXML.AppendFormat(@"<SEARCHCOLLECTION>
                                                    {0}", GetSessionInfoNode());
                if (xpNodItr_Collections.Count > 0)
                {
                    sb_SearchInputXML.AppendFormat(@"<SEARCH>
	                                                <TARGETOBJECT>DIGITALASSET</TARGETOBJECT>
	                                                <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
	                                                <SEARCHALLFACETS>FALSE</SEARCHALLFACETS>
	                                                <TARGETENTITY>
		                                                <PRIVILEGENAME>READ</PRIVILEGENAME>
		                                                <SELECTLIST>
			                                                <EXPRESSION>
				                                                <FIELDNAME>DIGITALASSETID</FIELDNAME>
				                                                <ALIAS>DIGITALASSETID</ALIAS>
			                                                </EXPRESSION>
		                                                </SELECTLIST>
		                                                <FILTERLIST>
			                                                <FILTER>
				                                                <EXPRESSION>COLLECTIONID</EXPRESSION>
				                                                <OPERATOR>IN</OPERATOR>
				                                                <EXPRESSIONVALUE>({0})</EXPRESSIONVALUE>
				                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>
				                                                <RELATIONSHIPNAME>COLLECTION_DIGITALASSETS</RELATIONSHIPNAME>
			                                                </FILTER>
		                                                </FILTERLIST>
		                                                <RELATIONSHIP NAME='SECTION_DIGITALASSET'>
                                                            <SELECTLIST>
			                                                    <EXPRESSION>
				                                                    <FIELDNAME>SECTIONID</FIELDNAME>
				                                                    <ALIAS>SECTIONID</ALIAS>
			                                                    </EXPRESSION>
		                                                    </SELECTLIST>   
			                                                <RELATIONSHIP NAME='SECTION_VAULT'>
				                                                <SELECTLIST>
					                                                <EXPRESSION>
						                                                <FIELDNAME>VAULTID</FIELDNAME>
						                                                <ALIAS>VAULTID</ALIAS>
					                                                </EXPRESSION>
					                                                <EXPRESSION>
						                                                <FIELDNAME>NAME</FIELDNAME>
						                                                <ALIAS>NAME</ALIAS>
					                                                </EXPRESSION>
					                                                <EXPRESSION>
						                                                <FIELDNAME>TERMSANDCONDITIONSTYPE</FIELDNAME>
						                                                <ALIAS>TERMSANDCONDITIONSTYPE</ALIAS>
					                                                </EXPRESSION>
					                                                <EXPRESSION>
						                                                <FIELDNAME>TERMSANDCONDITIONSFILE</FIELDNAME>
						                                                <ALIAS>TERMSANDCONDITIONSTYPE</ALIAS>
					                                                </EXPRESSION>
				                                                </SELECTLIST>
			                                                </RELATIONSHIP>
		                                                </RELATIONSHIP>
	                                                </TARGETENTITY>
                                                </SEARCH>", sb_CollectionIDList.ToString());
                }

                if (xpNodItr_DigitalAssets.Count > 0)
                {
                    sb_SearchInputXML.AppendFormat(@"<SEARCH>
	                                            	<TARGETOBJECT>SECTION</TARGETOBJECT>
	                                                <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
	                                                <SEARCHALLFACETS>FALSE</SEARCHALLFACETS>
	                                                <TARGETENTITY>
		                                                <PRIVILEGENAME>READ</PRIVILEGENAME>
		                                                <SELECTLIST>
			                                                <EXPRESSION>
				                                                <FIELDNAME>SECTIONID</FIELDNAME>
				                                                <ALIAS>SECTIONID</ALIAS>
			                                                </EXPRESSION>
		                                                </SELECTLIST>
		                                                <FILTERLIST>
			                                                <FILTER>
				                                                <EXPRESSION>DIGITALASSETID</EXPRESSION>
				                                                <OPERATOR>IN</OPERATOR>
				                                                <EXPRESSIONVALUE>({0})</EXPRESSIONVALUE>
				                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>
				                                                <FILTERID>1</FILTERID>
				                                                <RELATIONSHIPNAME>SECTION_DIGITALASSET</RELATIONSHIPNAME>
			                                                </FILTER>
		                                                </FILTERLIST>
		                                                <RELATIONSHIP NAME='SECTION_VAULT'>
				                                                <SELECTLIST>
					                                                <EXPRESSION>
						                                                <FIELDNAME>VAULTID</FIELDNAME>
						                                                <ALIAS>VAULTID</ALIAS>
					                                                </EXPRESSION>
                                                                     <EXPRESSION>   
						                                                <FIELDNAME>NAME</FIELDNAME>
						                                                <ALIAS>NAME</ALIAS>
					                                                </EXPRESSION>
					                                                <EXPRESSION>
						                                                <FIELDNAME>TERMSANDCONDITIONSTYPE</FIELDNAME>
						                                                <ALIAS>TERMSANDCONDITIONSTYPE</ALIAS>
					                                                </EXPRESSION>
                                                                    <EXPRESSION>
						                                                <FIELDNAME>TERMSANDCONDITIONSFILE</FIELDNAME>
						                                                <ALIAS>TERMSANDCONDITIONSTYPE</ALIAS>
					                                                </EXPRESSION>
				                                                </SELECTLIST>
		                                                </RELATIONSHIP>
	                                                </TARGETENTITY>
                                        </SEARCH>", sb_DAIDList.ToString());
                }

                sb_SearchInputXML.Append("</SEARCHCOLLECTION>");

                obj_SearchConfiguration = new SearchConfiguration(SessionID, UserID, TenantName, CurrentLanguage);
                ErrorXML                = obj_SearchConfiguration.GetHierarchialDataEx(sb_SearchInputXML.ToString(), ref SearchOutPutXML);
                if (!String.IsNullOrEmpty(ErrorXML))
                {
                    return ErrorXML;
                }
                
                xpDoc_VaultList         = new XPathDocument(new StringReader(SearchOutPutXML));
                xpNav_RootVaultList     = xpDoc_VaultList.CreateNavigator();

                sb_VaultListXML = new StringBuilder(100);
                xpNodItr_VaultList = xpNav_RootVaultList.Select("SEARCHOUTPUTXML/SECTIONCOLLECTION/SECTION/SECTION_VAULT/VAULT/VAULTID");
                foreach (XPathNavigator xpNav_Vault in xpNodItr_VaultList)
                {
                    if (!VaultList.Contains(xpNav_Vault.Value))
                    {
                        VaultList.Add(xpNav_Vault.Value);
                        xpNav_Vault.MoveToParent();
                        sb_VaultListXML.Append(xpNav_Vault.OuterXml);
                    }
                }

                xpNodItr_VaultList = xpNav_RootVaultList.Select("SEARCHOUTPUTXML/DIGITALASSETCOLLECTION/DIGITALASSET/SECTION_DIGITALASSET/SECTION/SECTION_VAULT/VAULT/VAULTID");
                foreach (XPathNavigator xpNav_Vault in xpNodItr_VaultList)
                {
                    if (!VaultList.Contains(xpNav_Vault.Value))
                    {
                        VaultList.Add(xpNav_Vault.Value);
                        xpNav_Vault.MoveToParent();
                        sb_VaultListXML.Append(xpNav_Vault.OuterXml);
                    }
                }
                sb_VaultListXML.Insert(0, "<VAULTCOLLECTION>");
                sb_VaultListXML.Append("</VAULTCOLLECTION>");
                VaultCollection     = sb_VaultListXML.ToString();
                xpDoc_VaultList     = new XPathDocument(new StringReader(sb_VaultListXML.ToString()));
                xpNav_RootVaultList = xpDoc_VaultList.CreateNavigator();
                xpNodItr_VaultList  = xpNav_RootVaultList.Select("VAULTCOLLECTION/VAULT/VAULTID");
                if (IsTermsAndConditionsEnabled)
                {
                    if (VaultList.Count == 1)
                    {
                        xpNav_Vault_TermsAndCSType = xpNav_RootVaultList.SelectSingleNode("VAULTCOLLECTION/VAULT/TERMSANDCONDITIONSTYPE[text()='0']");
                        if (xpNav_Vault_TermsAndCSType != null)
                        {
                            IsTandCSRequired = false;
                        }
                        xpNav_Vault_TermsAndCSType = xpNav_RootVaultList.SelectSingleNode("VAULTCOLLECTION/VAULT/TERMSANDCONDITIONSTYPE[text()='1']");
                        if (xpNav_Vault_TermsAndCSType != null)
                        {
                            isUsedDefaultTandCS = true;
                        }

                        VaultID = xpNav_RootVaultList.SelectSingleNode("VAULTCOLLECTION/VAULT/VAULTID").Value;
                        sb_FormName = new StringBuilder(100);
                        sb_FormName.Remove(0, sb_FormName.Length);
                        sb_FormName.AppendFormat("{0}_ReasonForDownload_FormConfiguration", VaultID);
                        str_FormConfigXMLPath = ALSolutionCommon.GetFormConfigurationXMLPath(TenantName, SolutionName, sb_FormName.ToString());
                        if (!String.IsNullOrEmpty(str_FormConfigXMLPath) && File.Exists(str_FormConfigXMLPath))
                        {
                            VaultIDForReasonForDownload = VaultID;
                        }
                        IsCrossVaultAllowed = true;
                        return null;
                    }
                    else
                    {
                        xpNav_Vault_TermsAndCSType = xpNav_RootVaultList.SelectSingleNode("VAULTCOLLECTION/VAULT/TERMSANDCONDITIONSTYPE[text()='2']");
                        if (xpNav_Vault_TermsAndCSType != null)
                        {
                            IsCrossVaultAllowed = false;
                            return null;
                        }
                        xpNav_Vault_TermsAndCSType = xpNav_RootVaultList.SelectSingleNode("VAULTCOLLECTION/VAULT/TERMSANDCONDITIONSTYPE[text()='1']");
                        if (xpNav_Vault_TermsAndCSType != null)
                            IsVaultUsingDefaultTandCS = true;
                        xpNav_Vault_TermsAndCSType = xpNav_RootVaultList.SelectSingleNode("VAULTCOLLECTION/VAULT/TERMSANDCONDITIONSTYPE[text()='0']");
                        if (xpNav_Vault_TermsAndCSType != null)
                            IsVaultNotRequiredTandCS = true;

                        //If one of the vault is using "default Terms and conditions" and One of them "not required Terms and Conditions": CrossvaultAllowed = false
                        if (IsVaultUsingDefaultTandCS && IsVaultNotRequiredTandCS)
                        {
                            IsTandCSRequired    = false;
                            IsCrossVaultAllowed = false;
                            return null;
                        }

                        //If none of the vault is using "default Terms and Conditions" i.e. all vaults "not required Terms and Conditions", then check for reason for download 
                        if (!IsVaultUsingDefaultTandCS && IsVaultNotRequiredTandCS)
                        {
                            IsTandCSRequired = false;
                        }

                        //If none of the vault is "not required Terms and Conditions" i.e. all vaults using "default Terms and Conditions", then check for reason for download 
                        if ((IsVaultUsingDefaultTandCS && !IsVaultNotRequiredTandCS))
                        {
                            isUsedDefaultTandCS = true;
                        }

                        if (IsReasonForDownloadEnabled)
                        {
                            sb_FormName = new StringBuilder(100);
                            foreach (XPathNavigator xpNav_Vault in xpNodItr_VaultList)
                            {
                                VaultID = xpNav_Vault.Value;
                                sb_FormName.Remove(0, sb_FormName.Length);
                                sb_FormName.AppendFormat("{0}_ReasonForDownload_FormConfiguration", VaultID);

                                str_FormConfigXMLPath = ALSolutionCommon.GetFormConfigurationXMLPath(TenantName, SolutionName, sb_FormName.ToString());
                                if (!String.IsNullOrEmpty(str_FormConfigXMLPath) && File.Exists(str_FormConfigXMLPath))
                                {
                                    IsCrossVaultAllowed = false;
                                    isVaultSpecificRFDUsed = true;
                                    return null;
                                }
                            }
                        }
                        IsCrossVaultAllowed = true;
                        return null;
                    }
                }
                else
                {
                    sb_FormName = new StringBuilder(100);
                    if (xpNodItr_VaultList.Count == 1)
                    {
                        VaultID = xpNav_RootVaultList.SelectSingleNode("VAULTCOLLECTION/VAULT/VAULTID").Value;
                        
                        sb_FormName.Remove(0, sb_FormName.Length);
                        sb_FormName.AppendFormat("{0}_ReasonForDownload_FormConfiguration", VaultID);
                        str_FormConfigXMLPath = ALSolutionCommon.GetFormConfigurationXMLPath(TenantName, SolutionName, sb_FormName.ToString());
                        if (!String.IsNullOrEmpty(str_FormConfigXMLPath) && File.Exists(str_FormConfigXMLPath))
                        {
                            VaultIDForReasonForDownload = VaultID;
                        }
                    }
                    else
                    {
                        foreach (XPathNavigator xpNav_Vault in xpNodItr_VaultList)
                        {
                            VaultID = xpNav_Vault.Value;
                            sb_FormName.Remove(0, sb_FormName.Length);
                            sb_FormName.AppendFormat("{0}_ReasonForDownload_FormConfiguration", VaultID);

                            str_FormConfigXMLPath = ALSolutionCommon.GetFormConfigurationXMLPath(TenantName, SolutionName, sb_FormName.ToString());
                            if (!String.IsNullOrEmpty(str_FormConfigXMLPath) && File.Exists(str_FormConfigXMLPath))
                            {
                                IsCrossVaultAllowed     = false;
                                isVaultSpecificRFDUsed  = true;
                                return null;
                            }
                        }
                    }
                    IsCrossVaultAllowed = true;
                    IsTandCSRequired    = false;
                    return null;
                }

               
                //return null;
            }
            catch (Exception obj_Exception)
            {
                return obj_Exception.Message;
            }
        }

        public ERRORXML GetAssetRenditions(String AssetIDList, out String SearchResultXML)
        {
            StringBuilder sb_SearchInputXML = null;
            SearchResultXML = String.Empty;
            try
            {
                sb_SearchInputXML = new StringBuilder(200);
                sb_SearchInputXML.AppendFormat(@"
                                            <SEARCH>
                                                {0}
	                                            <TARGETOBJECT>CONTENTREVISION</TARGETOBJECT>
	                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
	                                            <SEARCHALLFACETS>FALSE</SEARCHALLFACETS>
	                                            <TARGETENTITY>
		                                            <PRIVILEGENAME>READ</PRIVILEGENAME>
		                                            <SELECTLIST>
			                                            <EXPRESSION>
				                                            <FIELDNAME>CONTENTREVISIONID</FIELDNAME>
				                                            <ALIAS>CONTENTREVISIONID</ALIAS>
			                                            </EXPRESSION>
			                                            <EXPRESSION>
				                                            <FIELDNAME>VERSIONNUMBER</FIELDNAME>
				                                            <ALIAS>VERSIONNUMBER</ALIAS>
			                                            </EXPRESSION>
		                                            </SELECTLIST>
		                                            <FILTERLIST>
			                                            <FILTER>
				                                            <EXPRESSION>DIGITALASSETID</EXPRESSION>
				                                            <OPERATOR>IN</OPERATOR>
				                                            <EXPRESSIONVALUE>({1})</EXPRESSIONVALUE>
				                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
				                                            <FILTERID>1</FILTERID>
				                                            <RELATIONSHIPNAME>DIGITALASSET_CURRENTREVISION</RELATIONSHIPNAME>
			                                            </FILTER>
		                                            </FILTERLIST>
		                                            <RELATIONSHIP NAME='CONTENTREVISION_RENDITION'>
			                                            <SELECTLIST>
				                                            <EXPRESSION>
					                                            <FIELDNAME>ASSETRENDITIONID</FIELDNAME>
					                                            <ALIAS>ASSETRENDITIONID</ALIAS>
				                                            </EXPRESSION>
				                                            <EXPRESSION>
					                                            <FIELDNAME>NAME</FIELDNAME>
					                                            <ALIAS>NAME</ALIAS>
				                                            </EXPRESSION>
				                                            <EXPRESSION>
					                                            <FIELDNAME>RENDITIONTYPE</FIELDNAME>
					                                            <ALIAS>RENDITIONTYPE</ALIAS>
				                                            </EXPRESSION>
				                                            <EXPRESSION>
					                                            <FIELDNAME>FILENAME</FIELDNAME>
					                                            <ALIAS>FILENAME</ALIAS>
				                                            </EXPRESSION>
				                                            <EXPRESSION>
					                                            <FIELDNAME>FILESIZE</FIELDNAME>
					                                            <ALIAS>FILESIZE</ALIAS>
				                                            </EXPRESSION>
                                                            <EXPRESSION>
					                                            <FIELDNAME>FILEFORMAT</FIELDNAME>
					                                            <ALIAS>FILEFORMAT</ALIAS>
				                                            </EXPRESSION>
			                                            </SELECTLIST>
			                                            <FILTERLIST>
				                                            <FILTER>
					                                            <EXPRESSION>RENDITIONTYPE</EXPRESSION>
					                                            <OPERATOR>EQUALTO</OPERATOR>
					                                            <EXPRESSIONVALUE>Thumbnail</EXPRESSIONVALUE>
					                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
				                                            </FILTER>
				                                            <FILTER>
					                                            <EXPRESSION>RENDITIONTYPE</EXPRESSION>
					                                            <OPERATOR>EQUALTO</OPERATOR>
					                                            <EXPRESSIONVALUE>Preview</EXPRESSIONVALUE>
					                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
				                                            </FILTER>
				                                            <FILTER>
					                                            <EXPRESSION>RENDITIONTYPE</EXPRESSION>
					                                            <OPERATOR>EQUALTO</OPERATOR>
					                                            <EXPRESSIONVALUE>Original</EXPRESSIONVALUE>
					                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
				                                            </FILTER>
			                                            </FILTERLIST>
		                                            </RELATIONSHIP>
		                                            <RELATIONSHIP NAME='DIGITALASSET_CURRENTREVISION'>
			                                            <SELECTLIST>
				                                            <EXPRESSION>
					                                            <FIELDNAME>DIGITALASSETID</FIELDNAME>
					                                            <ALIAS>DIGITALASSETID</ALIAS>
				                                            </EXPRESSION>
				                                            <EXPRESSION>
					                                            <FIELDNAME>NAME</FIELDNAME>
					                                            <ALIAS>NAME</ALIAS>
				                                            </EXPRESSION>
                                                            <EXPRESSION>
						                                        <FIELDNAME>EXPIRYDATE</FIELDNAME>
						                                        <ALIAS>EXPIRYDATE</ALIAS>
					                                         </EXPRESSION>
			                                            </SELECTLIST>
			                                            <FILTERLIST>
				                                            <FILTER>
					                                            <EXPRESSION>DIGITALASSETID</EXPRESSION>
					                                            <OPERATOR>IN</OPERATOR>
					                                            <EXPRESSIONVALUE>({1})</EXPRESSIONVALUE>
					                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
					                                            <FILTERID>1</FILTERID>
				                                            </FILTER>
			                                            </FILTERLIST>
			                                            <RELATIONSHIP NAME='SECTION_DIGITALASSET'>
				                                            <SELECTLIST>
					                                            <EXPRESSION>
						                                            <FIELDNAME>SECTIONID</FIELDNAME>
						                                            <ALIAS>SECTIONID</ALIAS>
					                                            </EXPRESSION>
				                                            </SELECTLIST>
				                                            <RELATIONSHIP NAME='SECTION_VAULT'>
					                                            <SELECTLIST>
						                                            <EXPRESSION>
							                                            <FIELDNAME>VAULTID</FIELDNAME>
							                                            <ALIAS>VAULTID</ALIAS>
						                                            </EXPRESSION>
						                                            <EXPRESSION>
							                                            <FIELDNAME>NAME</FIELDNAME>
							                                            <ALIAS>NAME</ALIAS>
						                                            </EXPRESSION>
					                                            </SELECTLIST>
				                                            </RELATIONSHIP>
			                                            </RELATIONSHIP>
		                                            </RELATIONSHIP>
	                                            </TARGETENTITY>
                                            </SEARCH>", GetSessionInfoNode(), AssetIDList);
                SearchConfiguration obj_SearchConfiguration = new SearchConfiguration(SessionID, UserID, TenantName, CurrentLanguage);
                ErrorXML = obj_SearchConfiguration.GetHierarchialData(sb_SearchInputXML.ToString(), ref SearchResultXML);
                if (!String.IsNullOrEmpty(ErrorXML))
                {
                    return ErrorXML;
                }
                return null;
            }
            catch (Exception obj_Exception)
            {
                return obj_Exception.Message;
            }

        }

        public ERRORXML PrepareAndExecuteWSMethod(XPathNavigator xpNav_ActionBarXML,String ActionControlName,StringDictionary DAMParams)
        {
            XPathNavigator          xpNav_ActionControl         = null;
            String                  str_ActionControlType       = null;
            String                  str_SearchConfigName      = null;
            String                  EventName                   = null;
            String                  str_WebServiceConfigName    = null;
            DAMWebServiceHandler    DAMWebServiceHandlerObj     = null;
            ERRORXML                ErrorXML                    = null;
            
            try
            {
                xpNav_ActionControl     = xpNav_ActionBarXML.SelectSingleNode("ActionsCommon/ActionControlList/ActionControl[@Name='" + ActionControlName + "']");
                str_ActionControlType   = xpNav_ActionControl.GetAttribute("Type", "");

                if (String.Compare(str_ActionControlType, "WebService", true) == 0)
                {
                    str_WebServiceConfigName = xpNav_ActionControl.GetAttribute("WebServiceConfigName", "");
                    str_SearchConfigName     = xpNav_ActionControl.GetAttribute("SearchConfigName", "");
                    EventName                = xpNav_ActionControl.GetAttribute("EventName", "");

                    DAMWebServiceHandlerObj = new DAMWebServiceHandler(SessionID, UserID, TenantName, CurrentLanguage, SolutionName, ThemeName);
                    ErrorXML                = DAMWebServiceHandlerObj.InvokeWebService(str_WebServiceConfigName, str_SearchConfigName, EventName, DAMParams);
                    return ErrorXML;
                }
                else
                {
                    return DAMWebServiceHandler.PrepareErrorXML(TenantName,SolutionName, "", "ERROR", "", "Action control is not of WebService type","");
                }
            }
            catch (Exception ExceptionObj)
            {
                return DAMWebServiceHandler.PrepareErrorXML(TenantName,SolutionName, "", "ERROR", "", ExceptionObj.Message,"");
            }

            
        }

        public ERRORXML PrepareAndExecuteWSMethod(String ActionBarXMLPath, String ActionControlName, StringDictionary DAMParams)
        {
            XPathDocument   xpDoc_ActionBarXML = null;
            XPathNavigator  xpNav_ActionBarXML = null;
            ERRORXML        ErrorXML           = null;

            if (File.Exists(ActionBarXMLPath))
            {
                xpDoc_ActionBarXML = new XPathDocument(ActionBarXMLPath);
                xpNav_ActionBarXML = xpDoc_ActionBarXML.CreateNavigator();

                ErrorXML            = PrepareAndExecuteWSMethod(xpNav_ActionBarXML, ActionControlName, DAMParams);
                return ErrorXML;
            }
            else
            {
                return DAMWebServiceHandler.PrepareErrorXML(TenantName,SolutionName, "", "ERROR", "", "File does not exists","");
            }
        }

        public static ERRORXML ApplyXSLT(String XSLTPath,XPathNavigator xpNav,  String SourceXML, ref String OutputXML)
        {
            XslCompiledTransform    transformObj            = null;
            Stream                  MemStream               = null;
            XPathDocument           xpDoc                   = null;
            
            try
            {
                if (xpNav == null)
                {
                    xpDoc = new XPathDocument(new StringReader(SourceXML));
                    xpNav = xpDoc.CreateNavigator();
                }
                transformObj = new XslCompiledTransform();
                transformObj.Load(XSLTPath);

                MemStream    = new MemoryStream();
                transformObj.Transform(xpNav, null, MemStream);

                transformObj            = null;
                MemStream.Position      = 0;
                StreamReader sr         = new StreamReader(MemStream);
                String TransformedXML   = sr.ReadToEnd();
                TransformedXML          = TransformedXML.Replace("<?xml version=\"1.0\" encoding=\"utf-8\"?>", "");
                if (!String.IsNullOrEmpty(TransformedXML))
                {
                    OutputXML = TransformedXML;
                }
            }
            catch (Exception ExceptionObj)
            {
                return ExceptionObj.Message;
            }
            return null;
        }

        public String GetDownloadFileName(String str_FileFomat, String str_AssetName, String str_FileName, String str_RenditionType)
        {
            String str_DownloadFileName = null;
            str_DownloadFileName = ALCommon.GetDigitalAssetDownloadFileName(str_AssetName, str_FileFomat, 1, str_RenditionType, TenantName);
            return str_DownloadFileName;
        }

        /// <summary>
        /// This method is used to check whether the Asset name is proper or not
        /// If Actual Asset Name: Logo.jpg
        /// and User renamed it to Logo (removed the Extension) or Logo.Zip then while
        /// downloading asset name should be Logo.JPG or Logo.Zip.JPG
        /// </summary>
        /// <param name="AssetName"></param>
        /// <param name="OriginalFileFormat"></param>
        public void CheckAssetName(ref String AssetName, String OriginalFileFormat)
        {
            String str_FileExtension = null;
            String str_AssetWithoutExt = null;
            
            if (AssetName.IndexOf(".") != -1)
            {
                str_AssetWithoutExt = AssetName.Substring(0, AssetName.LastIndexOf("."));
                str_FileExtension = AssetName.Substring(AssetName.LastIndexOf(".") + 1);
            }
            else
                str_AssetWithoutExt = AssetName;

            //If Asset name does not contains any extension
            if (String.Compare(AssetName, str_AssetWithoutExt, true) == 0 && !String.IsNullOrEmpty(OriginalFileFormat))
                AssetName = String.Format("{0}.{1}", AssetName, OriginalFileFormat);
            else if (String.Compare(AssetName, str_AssetWithoutExt, true) != 0)
            {
                if (!String.IsNullOrEmpty(OriginalFileFormat) && !String.IsNullOrEmpty(str_FileExtension))
                {
                    switch (OriginalFileFormat)
                    {
                        case "JPEG":
                            {
                                if (String.Compare(str_FileExtension, "jpg", true) != 0
                                 && String.Compare(str_FileExtension, "jpeg", true) != 0
                                 && String.Compare(str_FileExtension, "jpe", true) != 0)
                                {
                                    AssetName = String.Format("{0}.{1}", AssetName, OriginalFileFormat);
                                }
                            } break;
                        case "HTM":
                            {
                                if (String.Compare(str_FileExtension, "htm", true) != 0
                                && String.Compare(str_FileExtension, "html", true) != 0)
                                {
                                    AssetName = String.Format("{0}.{1}", AssetName, OriginalFileFormat);
                                }
                            } break;
                        case "TEXT":
                            {
                                if (String.Compare(str_FileExtension, "txt", true) != 0
                                && String.Compare(str_FileExtension, "text", true) != 0)
                                {
                                    AssetName = String.Format("{0}.{1}", AssetName, OriginalFileFormat);
                                }
                            } break;
                        default:
                            if (String.Compare(str_FileExtension, OriginalFileFormat, true) != 0)
                                AssetName = String.Format("{0}.{1}", AssetName, OriginalFileFormat);
                            break;
                    }
                }
            }
        }
        #endregion
    }
}
